/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.common.constants;

import com.alibaba.polardbx.common.properties.ConnectionProperties;
import com.google.common.collect.ImmutableSet;

import java.util.HashSet;
import java.util.Set;

public class ServerVariables {

    public static Set<String> variables = new HashSet<String>();
    public static Set<String> readonlyVariables = new HashSet<String>();
    public static Set<String> writableVariables = new HashSet<String>();
    public static Set<String> bannedVariables = new HashSet<String>();
    public static Set<String> extraVariables = new HashSet<String>();
    public static Set<String> XbannedVariables = new HashSet<>();

    public static Set<String> canExecByBoth = new HashSet<String>();

    public static Set<String> canOnlyExecByNullVariables = new HashSet<String>();

    public static Set<String> canOnlyExecByEmptyStrVariables = new HashSet<String>();

    public static Set<String> mysqlBothVariables = new HashSet<>();
    public static Set<String> mysqlGlobalVariables = new HashSet<>();
    public static Set<String> mysqlSessionVariables = new HashSet<>();
    public static Set<String> mysqlDynamicVariables = new HashSet<>();

    public static Set<String> globalBannedVariables = new HashSet<>();

    public static final Set<String> SUPPORT_SET_GLOBAL_VARIABLES;

    /**
     * Parameters of timer task which are modifiable during runtime.
     */
    public static final Set<String> MODIFIABLE_PURGE_TRANS_PARAM = ImmutableSet.of(
        ConnectionProperties.PURGE_TRANS_INTERVAL,
        ConnectionProperties.PURGE_TRANS_BEFORE,
        ConnectionProperties.PURGE_TRANS_START_TIME);
    public static final Set<String> MODIFIABLE_DEADLOCK_DETECTION_PARAM = ImmutableSet.of(
        ConnectionProperties.ENABLE_DEADLOCK_DETECTION,
        ConnectionProperties.DEADLOCK_DETECTION_INTERVAL);

    public static final Set<String> MODIFIABLE_TIMER_TASK_PARAM;

    static {
        variables.add("audit_log_current_session");
        variables.add("audit_log_filter_id");
        variables.add("auto_increment_increment");
        variables.add("auto_increment_offset");
        variables.add("autocommit");
        variables.add("automatic_sp_privileges");
        variables.add("back_log");
        variables.add("basedir");
        variables.add("balancer_max_partition_size");
        variables.add("big_tables");
        variables.add("bind_address");
        variables.add("binlogging_impossible_mode");
        variables.add("binlog_cache_size");
        variables.add("binlog_checksum");
        variables.add("binlog_direct_non_transactional_updates");
        variables.add("binlog_format");
        variables.add("binlog_max_flush_queue_time");
        variables.add("binlog_order_commits");
        variables.add("binlog_row_image");
        variables.add("binlog_row_value_options");
        variables.add("binlog_rows_query_log_events");
        variables.add("binlog_stmt_cache_size");
        variables.add("block_encryption_mode");
        variables.add("bulk_insert_buffer_size");
        variables.add("character_set_client");
        variables.add("character_set_connection");
        variables.add("character_set_database");
        variables.add("character_set_filesystem");
        variables.add("character_set_results");
        variables.add("character_set_server");
        variables.add("character_set_system");
        variables.add("character_sets_dir");
        variables.add("check_proxy_users");
        variables.add("collation_connection");
        variables.add("collation_database");
        variables.add("collation_server");
        variables.add("completion_type");
        variables.add("concurrent_insert");
        variables.add("consensus_auto_leader_transfer");
        variables.add("consensus_auto_leader_transfer_check_seconds");
        variables.add("consensus_auto_reset_match_index");
        variables.add("connect_timeout");
        variables.add("core_file");
        variables.add("cte_max_recursion_depth");
        variables.add("datadir");
        variables.add("date_format");
        variables.add("datetime_format");
        variables.add("debug");
        variables.add("debug_sync");
        variables.add("default_collation_for_utf8mb4");
        variables.add("default_storage_engine");
        variables.add("default_table_encryption");
        variables.add("default_tmp_storage_engine");
        variables.add("default_week_format");
        variables.add("delay_key_write");
        variables.add("delayed_insert_limit");
        variables.add("delayed_insert_timeout");
        variables.add("delayed_queue_size");
        variables.add("disconnect_on_expired_password");
        variables.add("div_precision_increment");
        variables.add("end_markers_in_json");
        variables.add("enforce_gtid_consistency");
        variables.add("engine_condition_pushdown");
        variables.add("enable_balancer");
        variables.add("eq_range_index_dive_limit");
        variables.add("error_count");
        variables.add("event_scheduler");
        variables.add("expire_logs_days");
        variables.add("explicit_defaults_for_timestamp");
        variables.add("external_user");
        variables.add("flush");
        variables.add("flush_time");
        variables.add("foreign_key_checks");
        variables.add("ft_boolean_syntax");
        variables.add("ft_max_word_len");
        variables.add("ft_min_word_len");
        variables.add("ft_query_expansion_limit");
        variables.add("ft_stopword_file");
        variables.add("general_log");
        variables.add("general_log_file");
        variables.add("generated_random_password_length");
        variables.add("group_concat_max_len");
        variables.add("group_replication_consistency");
        variables.add("gtid_done");
        variables.add("gtid_executed");
        variables.add("gtid_mode");
        variables.add("gtid_next");
        variables.add("gtid_owned");
        variables.add("gtid_purged");
        variables.add("have_compress");
        variables.add("have_crypt");
        variables.add("have_dynamic_loading");
        variables.add("have_geometry");
        variables.add("have_openssl");
        variables.add("have_profiling");
        variables.add("have_query_cache");
        variables.add("have_rtree_keys");
        variables.add("have_ssl");
        variables.add("have_symlink");
        variables.add("histogram_generation_max_mem_size");
        variables.add("host_cache_size");
        variables.add("hostname");
        variables.add("identity");
        variables.add("ignore_builtin_innodb");
        variables.add("ignore_db_dirs");
        variables.add("immediate_server_version");
        variables.add("information_schema_stats_expiry");
        variables.add("init_connect");
        variables.add("init_file");
        variables.add("init_slave");
        variables.add("innodb_adaptive_flushing");
        variables.add("innodb_adaptive_flushing_lwm");
        variables.add("innodb_adaptive_hash_index");
        variables.add("innodb_adaptive_max_sleep_delay");
        variables.add("innodb_additional_mem_pool_size");
        variables.add("innodb_api_bk_commit_interval");
        variables.add("innodb_api_disable_rowlock");
        variables.add("innodb_api_enable_binlog");
        variables.add("innodb_api_enable_mdl");
        variables.add("innodb_api_trx_level");
        variables.add("innodb_autoextend_increment");
        variables.add("innodb_autoinc_lock_mode");
        variables.add("innodb_buffer_pool_dump_at_shutdown");
        variables.add("innodb_buffer_pool_dump_now");
        variables.add("innodb_buffer_pool_filename");
        variables.add("innodb_buffer_pool_instances");
        variables.add("innodb_buffer_pool_load_abort");
        variables.add("innodb_buffer_pool_load_at_startup");
        variables.add("innodb_buffer_pool_load_now");
        variables.add("innodb_buffer_pool_size");
        variables.add("innodb_change_buffer_max_size");
        variables.add("innodb_change_buffering");
        variables.add("innodb_checksum_algorithm");
        variables.add("innodb_checksums");
        variables.add("innodb_cmp_per_index_enabled");
        variables.add("innodb_commit_concurrency");
        variables.add("innodb_compression_failure_threshold_pct");
        variables.add("innodb_compression_level");
        variables.add("innodb_compression_pad_pct_max");
        variables.add("innodb_concurrency_tickets");
        variables.add("innodb_data_file_path");
        variables.add("innodb_data_file_purge");
        variables.add("innodb_data_file_purge_interval");
        variables.add("innodb_data_file_purge_max_size");
        variables.add("innodb_data_home_dir");
        variables.add("innodb_disable_sort_file_cache");
        variables.add("innodb_doublewrite");
        variables.add("innodb_fast_shutdown");
        variables.add("innodb_file_format");
        variables.add("innodb_file_format_check");
        variables.add("innodb_file_format_max");
        variables.add("innodb_file_per_table");
        variables.add("innodb_flush_log_at_timeout");
        variables.add("innodb_flush_log_at_trx_commit");
        variables.add("innodb_flush_method");
        variables.add("innodb_flush_neighbors");
        variables.add("innodb_flushing_avg_loops");
        variables.add("innodb_force_load_corrupted");
        variables.add("innodb_force_recovery");
        variables.add("innodb_ft_aux_table");
        variables.add("innodb_ft_cache_size");
        variables.add("innodb_ft_enable_diag_print");
        variables.add("innodb_ft_enable_stopword");
        variables.add("innodb_ft_max_token_size");
        variables.add("innodb_ft_min_token_size");
        variables.add("innodb_ft_num_word_optimize");
        variables.add("innodb_ft_result_cache_limit");
        variables.add("innodb_ft_server_stopword_table");
        variables.add("innodb_ft_sort_pll_degree");
        variables.add("innodb_ft_total_cache_size");
        variables.add("innodb_ft_user_stopword_table");
        variables.add("innodb_io_capacity");
        variables.add("innodb_io_capacity_max");
        variables.add("innodb_large_prefix");
        variables.add("innodb_lock_wait_timeout");
        variables.add("innodb_locks_unsafe_for_binlog");
        variables.add("innodb_log_buffer_size");
        variables.add("innodb_log_compressed_pages");
        variables.add("innodb_log_file_size");
        variables.add("innodb_log_files_in_group");
        variables.add("innodb_log_group_home_dir");
        variables.add("innodb_lru_scan_depth");
        variables.add("innodb_max_dirty_pages_pct");
        variables.add("innodb_max_dirty_pages_pct_lwm");
        variables.add("innodb_max_purge_lag");
        variables.add("innodb_max_purge_lag_delay");
        variables.add("innodb_mirrored_log_groups");
        variables.add("innodb_monitor_disable");
        variables.add("innodb_monitor_enable");
        variables.add("innodb_monitor_reset");
        variables.add("innodb_monitor_reset_all");
        variables.add("innodb_old_blocks_pct");
        variables.add("innodb_old_blocks_time");
        variables.add("innodb_online_alter_log_max_size");
        variables.add("innodb_open_files");
        variables.add("innodb_optimize_fulltext_only");
        variables.add("innodb_optimize_point_storage");
        variables.add("innodb_page_size");
        variables.add("innodb_parallel_read_threads");
        variables.add("innodb_print_all_deadlocks");
        variables.add("innodb_purge_batch_size");
        variables.add("innodb_purge_threads");
        variables.add("innodb_random_read_ahead");
        variables.add("innodb_read_ahead_threshold");
        variables.add("innodb_read_io_threads");
        variables.add("innodb_read_only");
        variables.add("innodb_replication_delay");
        variables.add("innodb_rollback_on_timeout");
        variables.add("innodb_rollback_segments");
        variables.add("innodb_sort_buffer_size");
        variables.add("innodb_spin_wait_delay");
        variables.add("innodb_stats_auto_recalc");
        variables.add("innodb_stats_method");
        variables.add("innodb_stats_on_metadata");
        variables.add("innodb_stats_persistent");
        variables.add("innodb_stats_persistent_sample_pages");
        variables.add("innodb_stats_sample_pages");
        variables.add("innodb_stats_transient_sample_pages");
        variables.add("innodb_strict_mode");
        variables.add("innodb_support_instant_add_column");
        variables.add("innodb_support_xa");
        variables.add("innodb_sync_array_size");
        variables.add("innodb_sync_spin_loops");
        variables.add("innodb_table_locks");
        variables.add("innodb_thread_concurrency");
        variables.add("innodb_thread_sleep_delay");
        variables.add("innodb_undo_directory");
        variables.add("innodb_undo_logs");
        variables.add("innodb_undo_tablespaces");
        variables.add("innodb_use_native_aio");
        variables.add("innodb_use_sys_malloc");
        variables.add("innodb_version");
        variables.add("innodb_write_io_threads");
        variables.add("insert_id");
        variables.add("interactive_timeout");
        variables.add("internal_tmp_mem_storage_engine");
        variables.add("join_buffer_size");
        variables.add("keep_files_on_create");
        variables.add("key_buffer_size");
        variables.add("key_cache_age_threshold");
        variables.add("key_cache_block_size");
        variables.add("key_cache_division_limit");
        variables.add("large_files_support");
        variables.add("large_page_size");
        variables.add("large_pages");
        variables.add("last_insert_id");
        variables.add("lc_messages");
        variables.add("lc_messages_dir");
        variables.add("lc_time_names");
        variables.add("lc-messages");
        variables.add("license");
        variables.add("local_infile");
        variables.add("lock_wait_timeout");
        variables.add("log_bin");
        variables.add("log_bin_basename");
        variables.add("log_bin_index");
        variables.add("log_bin_trust_function_creators");
        variables.add("log_bin_use_v1_row_events");
        variables.add("log_error");
        variables.add("log_output");
        variables.add("log_queries_not_using_indexes");
        variables.add("log_slave_updates");
        variables.add("log_slow_admin_statements");
        variables.add("log_slow_slave_statements");
        variables.add("log_throttle_queries_not_using_indexes");
        variables.add("log_warnings");
        variables.add("long_query_time");
        variables.add("low_priority_updates");
        variables.add("lower_case_file_system");
        variables.add("lower_case_table_names");
        variables.add("master_info_repository");
        variables.add("master_verify_checksum");
        variables.add("max_allowed_packet");
        variables.add("max_binlog_cache_size");
        variables.add("max_binlog_size");
        variables.add("max_binlog_stmt_cache_size");
        variables.add("max_connect_errors");
        variables.add("max_connections");
        variables.add("max_delayed_threads");
        variables.add("max_error_count");
        variables.add("max_execution_time");
        variables.add("max_heap_table_size");
        variables.add("max_insert_delayed_threads");
        variables.add("max_join_size");
        variables.add("max_length_for_sort_data");
        variables.add("max_prepared_stmt_count");
        variables.add("max_points_in_geometry");
        variables.add("max_relay_log_size");
        variables.add("max_seeks_for_key");
        variables.add("max_sort_length");
        variables.add("max_sp_recursion_depth");
        variables.add("max_statement_time");
        variables.add("max_tmp_tables");
        variables.add("max_user_connections");
        variables.add("max_write_lock_count");
        variables.add("metadata_locks_cache_size");
        variables.add("metadata_locks_hash_instances");
        variables.add("min_examined_row_limit");
        variables.add("multi_range_count");
        variables.add("myisam_data_pointer_size");
        variables.add("myisam_max_sort_file_size");
        variables.add("myisam_mmap_size");
        variables.add("myisam_recover_options");
        variables.add("myisam_repair_threads");
        variables.add("myisam_sort_buffer_size");
        variables.add("myisam_stats_method");
        variables.add("myisam_use_mmap");
        variables.add("mysqlx_read_timeout");
        variables.add("mysqlx_wait_timeout");
        variables.add("mysqlx_write_timeout");
        variables.add("named_pipe");
        variables.add("ndbinfo_max_bytes");
        variables.add("ndbinfo_max_rows");
        variables.add("ndbinfo_show_hidden");
        variables.add("ndbinfo_table_prefix");
        variables.add("ndb_autoincrement_prefetch_sz");
        variables.add("ndb_blob_read_batch_bytes");
        variables.add("ndb_blob_write_batch_bytes");
        variables.add("ndb_dbg_check_shares");
        variables.add("ndb_deferred_constraints");
        variables.add("ndb_force_send");
        variables.add("ndb_fully_replicated");
        variables.add("ndb_join_pushdown");
        variables.add("ndb_index_stat_cache_entries");
        variables.add("ndb_index_stat_enable");
        variables.add("ndb_index_stat_option");
        variables.add("ndb_index_stat_update_freq");
        variables.add("ndb_log_bin");
        variables.add("ndb_log_exclusive_reads");
        variables.add("ndb_row_checksum");
        variables.add("ndb_table_no_logging");
        variables.add("ndb_table_temporary");
        variables.add("ndb_use_copying_alter_table");
        variables.add("ndb_use_exact_count");
        variables.add("ndb_use_transactions");
        variables.add("ndb-allow-copying-alter-table");
        variables.add("net_buffer_length");
        variables.add("net_read_timeout");
        variables.add("net_retry_count");
        variables.add("net_write_timeout");
        variables.add("innodb_purge_history");
        variables.add("replica_read_timeout");
        variables.add("loose_innodb_prepare_wait_timeout");
        variables.add("innodb_prepare_wait_timeout");
        variables.add("new");
        variables.add("old");
        variables.add("old_alter_table");
        variables.add("old_passwords");
        variables.add("open_files_limit");
        variables.add("optimizer_join_cache_level");
        variables.add("optimizer_prune_level");
        variables.add("optimizer_search_depth");
        variables.add("optimizer_switch");
        variables.add("optimizer_trace");
        variables.add("optimizer_trace_features");
        variables.add("optimizer_trace_limit");
        variables.add("optimizer_trace_max_mem_size");
        variables.add("optimizer_trace_offset");
        variables.add("original_commit_timestamp");
        variables.add("original_server_version");
        variables.add("parser_max_mem_size");
        variables.add("performance_schema");
        variables.add("performance_schema_accounts_size");
        variables.add("performance_schema_digests_size");
        variables.add("performance_schema_events_stages_history_long_size");
        variables.add("performance_schema_events_stages_history_size");
        variables.add("performance_schema_events_statements_history_long_size");
        variables.add("performance_schema_events_statements_history_size");
        variables.add("performance_schema_events_waits_history_long_size");
        variables.add("performance_schema_events_waits_history_size");
        variables.add("performance_schema_hosts_size");
        variables.add("performance_schema_max_cond_classes");
        variables.add("performance_schema_max_cond_instances");
        variables.add("performance_schema_max_file_classes");
        variables.add("performance_schema_max_file_handles");
        variables.add("performance_schema_max_file_instances");
        variables.add("performance_schema_max_mutex_classes");
        variables.add("performance_schema_max_mutex_instances");
        variables.add("performance_schema_max_rwlock_classes");
        variables.add("performance_schema_max_rwlock_instances");
        variables.add("performance_schema_max_socket_classes");
        variables.add("performance_schema_max_socket_instances");
        variables.add("performance_schema_max_stage_classes");
        variables.add("performance_schema_max_statement_classes");
        variables.add("performance_schema_max_table_handles");
        variables.add("performance_schema_max_table_instances");
        variables.add("performance_schema_max_thread_classes");
        variables.add("performance_schema_max_thread_instances");
        variables.add("performance_schema_session_connect_attrs_size");
        variables.add("performance_schema_setup_actors_size");
        variables.add("performance_schema_setup_objects_size");
        variables.add("performance_schema_users_size");
        variables.add("pid_file");
        variables.add("plugin_dir");
        variables.add("port");
        variables.add("preload_buffer_size");
        variables.add("print_identified_with_as_hex");
        variables.add("profiling");
        variables.add("profiling_history_size");
        variables.add("protocol_version");
        variables.add("proxy_user");
        variables.add("pseudo_slave_mode");
        variables.add("pseudo_thread_id");
        variables.add("query_alloc_block_size");
        variables.add("query_cache_limit");
        variables.add("query_cache_min_res_unit");
        variables.add("query_cache_size");
        variables.add("query_cache_type");
        variables.add("query_cache_wlock_invalidate");
        variables.add("query_prealloc_size");
        variables.add("rand_seed1");
        variables.add("rand_seed2");
        variables.add("range_alloc_block_size");
        variables.add("range_optimizer_max_mem_size");
        variables.add("polarx_max_allowed_packet");
        variables.add("polarx_max_connections");
        variables.add("polarx_connect_timeout");
        variables.add("rbr_exec_mode");
        variables.add("read_buffer_size");
        variables.add("read_only");
        variables.add("read_rnd_buffer_size");
        variables.add("relay_log");
        variables.add("relay_log_basename");
        variables.add("relay_log_index");
        variables.add("relay_log_info_file");
        variables.add("relay_log_info_repository");
        variables.add("relay_log_purge");
        variables.add("relay_log_recovery");
        variables.add("relay_log_space_limit");
        variables.add("report_host");
        variables.add("report_password");
        variables.add("report_port");
        variables.add("report_user");
        variables.add("resultset_metadata");
        variables.add("rpl_stop_slave_timeout");
        variables.add("secondary_engine_cost_threshold");
        variables.add("secure_auth");
        variables.add("secure_file_priv");
        variables.add("server_id");
        variables.add("server_id_bits");
        variables.add("server_uuid");
        variables.add("session_track_gtids");
        variables.add("session_track_schema");
        variables.add("session_track_state_change");
        variables.add("session_track_transaction_info");
        variables.add("shared_memory");
        variables.add("shared_memory_base_name");
        variables.add("skip_external_locking");
        variables.add("skip_name_resolve");
        variables.add("skip_networking");
        variables.add("skip_show_database");
        variables.add("slave_allow_batching");
        variables.add("slave_checkpoint_group");
        variables.add("slave_checkpoint_period");
        variables.add("slave_compressed_protocol");
        variables.add("slave_exec_mode");
        variables.add("slave_load_tmpdir");
        variables.add("slave_max_allowed_packet");
        variables.add("slave_net_timeout");
        variables.add("slave_parallel_workers");
        variables.add("slave_pending_jobs_size_max");
        variables.add("slave_rows_search_algorithms");
        variables.add("slave_skip_errors");
        variables.add("slave_sql_verify_checksum");
        variables.add("slave_transaction_retries");
        variables.add("slave_type_conversions");
        variables.add("show_create_table_skip_secondary_engine");
        variables.add("show_create_table_verbosity");
        variables.add("show_old_temporals");
        variables.add("slow_launch_time");
        variables.add("slow_query_log");
        variables.add("slow_query_log_file");
        variables.add("socket");
        variables.add("sort_buffer_size");
        variables.add("sql_auto_is_null");
        variables.add("sql_big_selects");
        variables.add("sql_big_tables");
        variables.add("sql_buffer_result");
        variables.add("sql_log_bin");
        variables.add("sql_log_off");
        variables.add("sql_low_priority_updates");
        variables.add("sql_max_join_size");
        variables.add("sql_mode");
        variables.add("sql_notes");
        variables.add("sql_quote_show_create");
        variables.add("sql_require_primary_key");
        variables.add("sql_safe_updates");
        variables.add("sql_select_limit");
        variables.add("sql_slave_skip_counter");
        variables.add("sql_warnings");
        variables.add("sql-mode");
        variables.add("ssl_ca");
        variables.add("ssl_capath");
        variables.add("ssl_cert");
        variables.add("ssl_cipher");
        variables.add("ssl_crl");
        variables.add("ssl_crlpath");
        variables.add("ssl_key");
        variables.add("storage_engine");
        variables.add("stored_program_cache");
        variables.add("sync_binlog");
        variables.add("sync_frm");
        variables.add("sync_master_info");
        variables.add("sync_relay_log");
        variables.add("sync_relay_log_info");
        variables.add("system_time_zone");
        variables.add("table_definition_cache");
        variables.add("table_open_cache");
        variables.add("table_open_cache_instances");
        variables.add("thread_cache_size");
        variables.add("thread_concurrency");
        variables.add("thread_handling");
        variables.add("thread_pool_high_priority_connection");
        variables.add("thread_pool_prio_kickup_timer");
        variables.add("thread_stack");
        variables.add("time_format");
        variables.add("time_zone");
        variables.add("timed_mutexes");
        variables.add("timestamp");
        variables.add("tmp_table_size");
        variables.add("tmpdir");
        variables.add("transaction_alloc_block_size");
        variables.add("transaction_allow_batching");
        variables.add("transaction_prealloc_size");
        variables.add("transaction_write_set_extraction");
        variables.add("tx_isolation");
        variables.add("transaction_isolation");
        variables.add("tx_read_only");
        variables.add("transaction_read_only");
        variables.add("unique_checks");
        variables.add("updatable_views_with_limit");
        variables.add("use_secondary_engine");
        variables.add("version");
        variables.add("version_comment");
        variables.add("version_compile_machine");
        variables.add("version_compile_os");
        variables.add("version_tokens_session");
        variables.add("version_tokens_session_number");
        variables.add("wait_timeout");
        variables.add("warning_count");
        variables.add("windowing_use_high_precision");
        variables.add("hotspot");
        variables.add("hotspot_lock_type");
        variables.add("hotspot_for_autocommit");
        variables.add("hotspot_update_max_wait_time");
        variables.add("innodb_hotspot_kill_lock_holder");
        variables.add("innodb_concurrency_tickets_hotspot");
        variables.add("innodb_hotspot_lock_wait_timeout");

        readonlyVariables.add("audit_log_current_session");
        readonlyVariables.add("audit_log_filter_id");
        readonlyVariables.add("back_log");
        readonlyVariables.add("basedir");
        readonlyVariables.add("bind_address");
        readonlyVariables.add("character_set_system");
        readonlyVariables.add("character_sets_dir");
        readonlyVariables.add("core_file");
        readonlyVariables.add("datadir");
        readonlyVariables.add("date_format");
        readonlyVariables.add("datetime_format");
        readonlyVariables.add("disconnect_on_expired_password");
        readonlyVariables.add("enforce_gtid_consistency");
        readonlyVariables.add("error_count");
        readonlyVariables.add("explicit_defaults_for_timestamp");
        readonlyVariables.add("external_user");
        readonlyVariables.add("ft_max_word_len");
        readonlyVariables.add("ft_min_word_len");
        readonlyVariables.add("ft_query_expansion_limit");
        readonlyVariables.add("ft_stopword_file");
        readonlyVariables.add("gtid_executed");
        readonlyVariables.add("gtid_done");
        readonlyVariables.add("gtid_mode");
        readonlyVariables.add("gtid_owned");
        readonlyVariables.add("have_compress");
        readonlyVariables.add("have_crypt");
        readonlyVariables.add("have_dynamic_loading");
        readonlyVariables.add("have_geometry");
        readonlyVariables.add("have_openssl");
        readonlyVariables.add("have_profiling");
        readonlyVariables.add("have_query_cache");
        readonlyVariables.add("have_rtree_keys");
        readonlyVariables.add("have_ssl");
        readonlyVariables.add("have_symlink");
        readonlyVariables.add("hostname");
        readonlyVariables.add("ignore_builtin_innodb");
        readonlyVariables.add("ignore_db_dirs");
        readonlyVariables.add("init_file");
        readonlyVariables.add("innodb_additional_mem_pool_size");
        readonlyVariables.add("innodb_api_disable_rowlock");
        readonlyVariables.add("innodb_api_enable_binlog");
        readonlyVariables.add("innodb_api_enable_mdl");
        readonlyVariables.add("innodb_autoinc_lock_mode");
        readonlyVariables.add("innodb_buffer_pool_instances");
        readonlyVariables.add("innodb_buffer_pool_load_at_startup");
        readonlyVariables.add("innodb_buffer_pool_size");
        readonlyVariables.add("innodb_checksums");
        readonlyVariables.add("innodb_data_file_path");
        readonlyVariables.add("innodb_data_home_dir");
        readonlyVariables.add("innodb_doublewrite");
        readonlyVariables.add("innodb_file_format_check");
        readonlyVariables.add("innodb_flush_method");
        readonlyVariables.add("innodb_force_load_corrupted");
        readonlyVariables.add("innodb_force_recovery");
        readonlyVariables.add("innodb_ft_cache_size");
        readonlyVariables.add("innodb_ft_max_token_size");
        readonlyVariables.add("innodb_ft_min_token_size");
        readonlyVariables.add("innodb_ft_sort_pll_degree");
        readonlyVariables.add("innodb_ft_total_cache_size");
        readonlyVariables.add("innodb_locks_unsafe_for_binlog");
        readonlyVariables.add("innodb_log_buffer_size");
        readonlyVariables.add("innodb_log_file_size");
        readonlyVariables.add("innodb_log_files_in_group");
        readonlyVariables.add("innodb_log_group_home_dir");
        readonlyVariables.add("innodb_mirrored_log_groups");
        readonlyVariables.add("innodb_open_files");
        readonlyVariables.add("innodb_page_size");
        readonlyVariables.add("innodb_purge_threads");
        readonlyVariables.add("innodb_read_io_threads");
        readonlyVariables.add("innodb_read_only");
        readonlyVariables.add("innodb_rollback_on_timeout");
        readonlyVariables.add("innodb_sort_buffer_size");
        readonlyVariables.add("innodb_sync_array_size");
        readonlyVariables.add("innodb_undo_directory");
        readonlyVariables.add("innodb_undo_tablespaces");
        readonlyVariables.add("innodb_use_native_aio");
        readonlyVariables.add("innodb_use_sys_malloc");
        readonlyVariables.add("innodb_version");
        readonlyVariables.add("innodb_write_io_threads");
        readonlyVariables.add("large_files_support");
        readonlyVariables.add("large_page_size");
        readonlyVariables.add("large_pages");
        readonlyVariables.add("lc_messages_dir");
        readonlyVariables.add("license");
        readonlyVariables.add("log_bin");
        readonlyVariables.add("log_bin_basename");
        readonlyVariables.add("log_bin_index");
        readonlyVariables.add("log_error");
        readonlyVariables.add("log_slave_updates");
        readonlyVariables.add("lower_case_file_system");
        readonlyVariables.add("lower_case_table_names");
        readonlyVariables.add("max_allowed_packet");
        readonlyVariables.add("max_user_connections");
        readonlyVariables.add("metadata_locks_cache_size");
        readonlyVariables.add("metadata_locks_hash_instances");
        readonlyVariables.add("myisam_mmap_size");
        readonlyVariables.add("myisam_recover_options");
        readonlyVariables.add("named_pipe");
        readonlyVariables.add("ndb_use_copying_alter_table");
        readonlyVariables.add("net_buffer_length");
        readonlyVariables.add("old");
        readonlyVariables.add("open_files_limit");
        readonlyVariables.add("performance_schema");
        readonlyVariables.add("performance_schema_accounts_size");
        readonlyVariables.add("performance_schema_digests_size");
        readonlyVariables.add("performance_schema_events_stages_history_long_size");
        readonlyVariables.add("performance_schema_events_stages_history_size");
        readonlyVariables.add("performance_schema_events_statements_history_long_size");
        readonlyVariables.add("performance_schema_events_statements_history_size");
        readonlyVariables.add("performance_schema_events_waits_history_long_size");
        readonlyVariables.add("performance_schema_events_waits_history_size");
        readonlyVariables.add("performance_schema_hosts_size");
        readonlyVariables.add("performance_schema_max_cond_classes");
        readonlyVariables.add("performance_schema_max_cond_instances");
        readonlyVariables.add("performance_schema_max_file_classes");
        readonlyVariables.add("performance_schema_max_file_handles");
        readonlyVariables.add("performance_schema_max_file_instances");
        readonlyVariables.add("performance_schema_max_mutex_classes");
        readonlyVariables.add("performance_schema_max_mutex_instances");
        readonlyVariables.add("performance_schema_max_rwlock_classes");
        readonlyVariables.add("performance_schema_max_rwlock_instances");
        readonlyVariables.add("performance_schema_max_socket_classes");
        readonlyVariables.add("performance_schema_max_socket_instances");
        readonlyVariables.add("performance_schema_max_stage_classes");
        readonlyVariables.add("performance_schema_max_statement_classes");
        readonlyVariables.add("performance_schema_max_table_handles");
        readonlyVariables.add("performance_schema_max_table_instances");
        readonlyVariables.add("performance_schema_max_thread_classes");
        readonlyVariables.add("performance_schema_max_thread_instances");
        readonlyVariables.add("performance_schema_session_connect_attrs_size");
        readonlyVariables.add("performance_schema_setup_actors_size");
        readonlyVariables.add("performance_schema_setup_objects_size");
        readonlyVariables.add("performance_schema_users_size");
        readonlyVariables.add("pid_file");
        readonlyVariables.add("plugin_dir");
        readonlyVariables.add("port");
        readonlyVariables.add("protocol_version");
        readonlyVariables.add("proxy_user");
        readonlyVariables.add("primary_zone");
        readonlyVariables.add("relay_log");
        readonlyVariables.add("relay_log_basename");
        readonlyVariables.add("relay_log_index");
        readonlyVariables.add("relay_log_info_file");
        readonlyVariables.add("relay_log_recovery");
        readonlyVariables.add("relay_log_space_limit");
        readonlyVariables.add("report_host");
        readonlyVariables.add("report_password");
        readonlyVariables.add("report_port");
        readonlyVariables.add("report_user");
        readonlyVariables.add("secure_file_priv");
        readonlyVariables.add("server_uuid");
        readonlyVariables.add("shared_memory");
        readonlyVariables.add("shared_memory_base_name");
        readonlyVariables.add("skip_external_locking");
        readonlyVariables.add("skip_name_resolve");
        readonlyVariables.add("skip_networking");
        readonlyVariables.add("skip_show_database");
        readonlyVariables.add("slave_load_tmpdir");
        readonlyVariables.add("slave_skip_errors");
        readonlyVariables.add("socket");
        readonlyVariables.add("ssl_ca");
        readonlyVariables.add("ssl_capath");
        readonlyVariables.add("ssl_cert");
        readonlyVariables.add("ssl_cipher");
        readonlyVariables.add("ssl_crl");
        readonlyVariables.add("ssl_crlpath");
        readonlyVariables.add("ssl_key");
        readonlyVariables.add("system_time_zone");
        readonlyVariables.add("table_open_cache_instances");
        readonlyVariables.add("thread_concurrency");
        readonlyVariables.add("thread_handling");
        readonlyVariables.add("thread_stack");
        readonlyVariables.add("time_format");
        readonlyVariables.add("tmpdir");
        readonlyVariables.add("version");
        readonlyVariables.add("version_comment");
        readonlyVariables.add("version_compile_machine");
        readonlyVariables.add("version_compile_os");
        readonlyVariables.add("warning_count");
        readonlyVariables.add("version_tokens_session_number");

        writableVariables.add("auto_increment_increment");
        writableVariables.add("auto_increment_offset");
        writableVariables.add("autocommit");
        writableVariables.add("big_tables");
        writableVariables.add("binlogging_impossible_mode");
        writableVariables.add("binlog_direct_non_transactional_updates");
        writableVariables.add("binlog_format");
        writableVariables.add("binlog_row_image");
        writableVariables.add("binlog_row_value_options");
        writableVariables.add("binlog_rows_query_log_events");
        writableVariables.add("block_encryption_mode");
        writableVariables.add("bulk_insert_buffer_size");
        writableVariables.add("character_set_client");
        writableVariables.add("character_set_connection");
        writableVariables.add("character_set_database");
        writableVariables.add("character_set_filesystem");
        writableVariables.add("character_set_results");
        writableVariables.add("character_set_server");
        writableVariables.add("collation_connection");
        writableVariables.add("collation_database");
        writableVariables.add("collation_server");
        writableVariables.add("completion_type");
        writableVariables.add("cte_max_recursion_depth");
        writableVariables.add("debug");
        writableVariables.add("debug_sync");
        writableVariables.add("default_collation_for_utf8mb4");
        writableVariables.add("default_storage_engine");
        writableVariables.add("default_table_encryption");
        writableVariables.add("default_tmp_storage_engine");
        writableVariables.add("default_week_format");
        writableVariables.add("div_precision_increment");
        writableVariables.add("end_markers_in_json");
        writableVariables.add("engine_condition_pushdown");
        writableVariables.add("eq_range_index_dive_limit");
        writableVariables.add("foreign_key_checks");
        writableVariables.add("generated_random_password_length");
        writableVariables.add("group_concat_max_len");
        writableVariables.add("group_replication_consistency");
        writableVariables.add("gtid_next");
        writableVariables.add("histogram_generation_max_mem_size");
        writableVariables.add("identity");
        writableVariables.add("immediate_server_version");
        writableVariables.add("information_schema_stats_expiry");
        writableVariables.add("innodb_ft_enable_stopword");
        writableVariables.add("innodb_ft_user_stopword_table");
        writableVariables.add("innodb_lock_wait_timeout");
        writableVariables.add("innodb_optimize_point_storage");
        writableVariables.add("innodb_parallel_read_threads");
        writableVariables.add("innodb_strict_mode");
        writableVariables.add("innodb_support_xa");
        writableVariables.add("innodb_table_locks");
        writableVariables.add("insert_id");
        writableVariables.add("interactive_timeout");
        writableVariables.add("internal_tmp_mem_storage_engine");
        writableVariables.add("join_buffer_size");
        writableVariables.add("keep_files_on_create");
        writableVariables.add("last_insert_id");
        writableVariables.add("lc_messages");
        writableVariables.add("lc_time_names");
        writableVariables.add("lc-messages");
        writableVariables.add("lock_wait_timeout");
        writableVariables.add("long_query_time");
        writableVariables.add("low_priority_updates");
        writableVariables.add("max_delayed_threads");
        writableVariables.add("max_error_count");
        writableVariables.add("max_execution_time");
        writableVariables.add("max_heap_table_size");
        writableVariables.add("max_insert_delayed_threads");
        writableVariables.add("max_join_size");
        writableVariables.add("max_length_for_sort_data");
        writableVariables.add("max_points_in_geometry");
        writableVariables.add("max_seeks_for_key");
        writableVariables.add("max_sort_length");
        writableVariables.add("max_sp_recursion_depth");
        writableVariables.add("max_statement_time");
        writableVariables.add("max_tmp_tables");
        writableVariables.add("min_examined_row_limit");
        writableVariables.add("multi_range_count");
        writableVariables.add("myisam_repair_threads");
        writableVariables.add("myisam_sort_buffer_size");
        writableVariables.add("myisam_stats_method");
        writableVariables.add("mysqlx_read_timeout");
        writableVariables.add("mysqlx_wait_timeout");
        writableVariables.add("mysqlx_write_timeout");
        writableVariables.add("ndbinfo_max_bytes");
        writableVariables.add("ndbinfo_max_rows");
        writableVariables.add("ndbinfo_show_hidden");
        writableVariables.add("ndbinfo_table_prefix");
        writableVariables.add("ndb_autoincrement_prefetch_sz");
        writableVariables.add("ndb_blob_read_batch_bytes");
        writableVariables.add("ndb_blob_write_batch_bytes");
        writableVariables.add("ndb_dbg_check_shares");
        writableVariables.add("ndb_deferred_constraints");
        writableVariables.add("ndb_fully_replicated");
        writableVariables.add("ndb_force_send");
        writableVariables.add("ndb_index_stat_cache_entries");
        writableVariables.add("ndb_index_stat_enable");
        writableVariables.add("ndb_index_stat_option");
        writableVariables.add("ndb_index_stat_update_freq");
        writableVariables.add("ndb_join_pushdown");
        writableVariables.add("ndb_log_bin");
        writableVariables.add("ndb_log_exclusive_reads");
        writableVariables.add("ndb_row_checksum");
        writableVariables.add("ndb_table_no_logging");
        writableVariables.add("ndb_table_temporary");
        writableVariables.add("ndb_use_exact_count");
        writableVariables.add("ndb_use_transactions");
        writableVariables.add("ndb-allow-copying-alter-table");
        writableVariables.add("net_read_timeout");
        writableVariables.add("net_retry_count");
        writableVariables.add("net_write_timeout");
        writableVariables.add("new");
        writableVariables.add("old_alter_table");
        writableVariables.add("old_passwords");
        writableVariables.add("optimizer_join_cache_level");
        writableVariables.add("optimizer_prune_level");
        writableVariables.add("optimizer_search_depth");
        writableVariables.add("optimizer_switch");
        writableVariables.add("optimizer_trace");
        writableVariables.add("optimizer_trace_features");
        writableVariables.add("optimizer_trace_limit");
        writableVariables.add("optimizer_trace_max_mem_size");
        writableVariables.add("optimizer_trace_offset");
        writableVariables.add("original_commit_timestamp");
        writableVariables.add("original_server_version");
        writableVariables.add("parser_max_mem_size");
        writableVariables.add("preload_buffer_size");
        writableVariables.add("print_identified_with_as_hex");
        writableVariables.add("profiling");
        writableVariables.add("profiling_history_size");
        writableVariables.add("pseudo_slave_mode");
        writableVariables.add("pseudo_thread_id");
        writableVariables.add("query_alloc_block_size");
        writableVariables.add("query_cache_wlock_invalidate");
        writableVariables.add("query_prealloc_size");
        writableVariables.add("rand_seed1");
        writableVariables.add("rand_seed2");
        writableVariables.add("range_alloc_block_size");
        writableVariables.add("range_optimizer_max_mem_size");
        writableVariables.add("polarx_max_connections");
        writableVariables.add("polarx_max_allowed_packet");
        writableVariables.add("polarx_connect_timeout");
        writableVariables.add("rbr_exec_mode");
        writableVariables.add("read_buffer_size");
        writableVariables.add("read_rnd_buffer_size");
        writableVariables.add("resultset_metadata");
        writableVariables.add("secondary_engine_cost_threshold");
        writableVariables.add("session_track_gtids");
        writableVariables.add("session_track_schema");
        writableVariables.add("session_track_state_change");
        writableVariables.add("session_track_transaction_info");
        writableVariables.add("show_create_table_skip_secondary_engine");
        writableVariables.add("show_create_table_verbosity");
        writableVariables.add("show_old_temporals");
        writableVariables.add("sort_buffer_size");
        writableVariables.add("sql_auto_is_null");
        writableVariables.add("sql_big_selects");
        writableVariables.add("sql_big_tables");
        writableVariables.add("sql_buffer_result");
        writableVariables.add("sql_log_bin");
        writableVariables.add("sql_log_off");
        writableVariables.add("sql_low_priority_updates");
        writableVariables.add("sql_max_join_size");
        writableVariables.add("sql_mode");
        writableVariables.add("sql_notes");
        writableVariables.add("sql_quote_show_create");
        writableVariables.add("sql_require_primary_key");
        writableVariables.add("sql_safe_updates");
        writableVariables.add("sql_select_limit");
        writableVariables.add("sql_warnings");
        writableVariables.add("sql-mode");
        writableVariables.add("storage_engine");
        writableVariables.add("thread_pool_high_priority_connection");
        writableVariables.add("thread_pool_prio_kickup_timer");
        writableVariables.add("time_zone");
        writableVariables.add("timestamp");
        writableVariables.add("tmp_table_size");
        writableVariables.add("transaction_alloc_block_size");
        writableVariables.add("transaction_allow_batching");
        writableVariables.add("transaction_prealloc_size");
        writableVariables.add("transaction_write_set_extraction");
        writableVariables.add("tx_isolation");
        writableVariables.add("transaction_isolation");
        writableVariables.add("tx_read_only");
        writableVariables.add("transaction_read_only");
        writableVariables.add("unique_checks");
        writableVariables.add("updatable_views_with_limit");
        writableVariables.add("use_secondary_engine");
        writableVariables.add("version_tokens_session");
        writableVariables.add("wait_timeout");
        writableVariables.add("windowing_use_high_precision");

        writableVariables.add("query_cache_type");

        bannedVariables.add("pseudo_thread_id");

        canExecByBoth.add("session_track_system_variables");

        canOnlyExecByNullVariables.add("character_set_results");
        canOnlyExecByNullVariables.add("innodb_ft_user_stopword_table");
        canOnlyExecByNullVariables.add("innodb_tmpdir");

        canOnlyExecByEmptyStrVariables.add("optimizer_switch");
        canOnlyExecByEmptyStrVariables.add("optimizer_trace");
        canOnlyExecByEmptyStrVariables.add("optimizer_trace_features");
        canOnlyExecByEmptyStrVariables.add("sql_mode");

        extraVariables.add(TransactionAttribute.DRDS_TRANSACTION_POLICY);

        variables.add(ConnectionProperties.MAX_TRX_DURATION.toLowerCase());
        writableVariables.add(ConnectionProperties.MAX_TRX_DURATION.toLowerCase());
        extraVariables.add(ConnectionProperties.MAX_TRX_DURATION.toLowerCase());

        extraVariables.add("sockettimeout");
        extraVariables.add("pure_async_ddl_mode");
        extraVariables.add("transaction policy");
        extraVariables.add("trans.policy");
        extraVariables.add("read");
        extraVariables.add("batch_insert_policy");
        extraVariables.add("sql_mock");
        extraVariables.add("primary_zone");
        extraVariables.add(ConnectionProperties.ENABLE_BALANCER.toLowerCase());
        extraVariables.add(ConnectionProperties.BALANCER_MAX_PARTITION_SIZE.toLowerCase());

        XbannedVariables.add("net_read_timeout");
        XbannedVariables.add("net_write_timeout");
        XbannedVariables.add("net_retry_count");

        mysqlBothVariables.add("audit_log_current_session");
        mysqlBothVariables.add("audit_log_filter_id");
        mysqlBothVariables.add("auto_increment_increment");
        mysqlBothVariables.add("auto_increment_offset");
        mysqlBothVariables.add("autocommit");
        mysqlBothVariables.add("big_tables");
        mysqlBothVariables.add("binlog_direct_non_transactional_updates");
        mysqlBothVariables.add("binlog_format");
        mysqlBothVariables.add("binlog_row_image");
        mysqlBothVariables.add("binlog_row_value_options");
        mysqlBothVariables.add("binlog_rows_query_log_events");
        mysqlBothVariables.add("binlog_transaction_compression");
        mysqlBothVariables.add("binlog_transaction_compression_level_zstd");
        mysqlBothVariables.add("block_encryption_mode");
        mysqlBothVariables.add("bulk_insert_buffer_size");
        mysqlBothVariables.add("character_set_client");
        mysqlBothVariables.add("character_set_connection");
        mysqlBothVariables.add("character_set_database(note 1)");
        mysqlBothVariables.add("character_set_filesystem");
        mysqlBothVariables.add("character_set_results");
        mysqlBothVariables.add("character_set_server");
        mysqlBothVariables.add("collation_connection");
        mysqlBothVariables.add("collation_database (note 1)");
        mysqlBothVariables.add("collation_server");
        mysqlBothVariables.add("completion_type");
        mysqlBothVariables.add("cte_max_recursion_depth");
        mysqlBothVariables.add("debug");
        mysqlBothVariables.add("default_collation_for_utf8mb4");
        mysqlBothVariables.add("default_storage_engine");
        mysqlBothVariables.add("default_table_encryption");
        mysqlBothVariables.add("default_tmp_storage_engine");
        mysqlBothVariables.add("default_week_format");
        mysqlBothVariables.add("div_precision_increment");
        mysqlBothVariables.add("end_markers_in_json");
        mysqlBothVariables.add("eq_range_index_dive_limit");
        mysqlBothVariables.add("explicit_defaults_for_timestamp");
        mysqlBothVariables.add("foreign_key_checks");
        mysqlBothVariables.add("generated_random_password_length");
        mysqlBothVariables.add("group_concat_max_len");
        mysqlBothVariables.add("group_replication_consistency");
        mysqlBothVariables.add("gtid_owned");
        mysqlBothVariables.add("histogram_generation_max_mem_size");
        mysqlBothVariables.add("information_schema_stats_expiry");
        mysqlBothVariables.add("innodb_ft_enable_stopword");
        mysqlBothVariables.add("innodb_ft_user_stopword_table");
        mysqlBothVariables.add("innodb_lock_wait_timeout");
        mysqlBothVariables.add("innodb_strict_mode");
        mysqlBothVariables.add("innodb_table_locks");
        mysqlBothVariables.add("innodb_tmpdir");
        mysqlBothVariables.add("interactive_timeout");
        mysqlBothVariables.add("internal_tmp_mem_storage_engine");
        mysqlBothVariables.add("join_buffer_size");
        mysqlBothVariables.add("keep_files_on_create");
        mysqlBothVariables.add("lc_messages");
        mysqlBothVariables.add("lc_time_names");
        mysqlBothVariables.add("lock_wait_timeout");
        mysqlBothVariables.add("long_query_time");
        mysqlBothVariables.add("low_priority_updates");
        mysqlBothVariables.add("max_allowed_packet");
        mysqlBothVariables.add("max_delayed_threads");
        mysqlBothVariables.add("max_error_count");
        mysqlBothVariables.add("max_execution_time");
        mysqlBothVariables.add("max_heap_table_size");
        mysqlBothVariables.add("max_insert_delayed_threads");
        mysqlBothVariables.add("max_join_size");
        mysqlBothVariables.add("max_length_for_sort_data");
        mysqlBothVariables.add("max_points_in_geometry");
        mysqlBothVariables.add("max_seeks_for_key");
        mysqlBothVariables.add("max_sort_length");
        mysqlBothVariables.add("max_sp_recursion_depth");
        mysqlBothVariables.add("max_user_connections");
        mysqlBothVariables.add("min_examined_row_limit");
        mysqlBothVariables.add("myisam_repair_threads");
        mysqlBothVariables.add("myisam_sort_buffer_size");
        mysqlBothVariables.add("myisam_stats_method");
        mysqlBothVariables.add("ndb_allow_copying_alter_table");
        mysqlBothVariables.add("ndb_autoincrement_prefetch_sz");
        mysqlBothVariables.add("ndb_blob_read_batch_bytes");
        mysqlBothVariables.add("ndb_blob_write_batch_bytes");
        mysqlBothVariables.add("ndb_dbg_check_shares");
        mysqlBothVariables.add("ndb_deferred_constraints");
        mysqlBothVariables.add("ndb_deferred_constraints");
        mysqlBothVariables.add("ndb_force_send");
        mysqlBothVariables.add("ndb_fully_replicated");
        mysqlBothVariables.add("ndb_index_stat_enable");
        mysqlBothVariables.add("ndb_index_stat_option");
        mysqlBothVariables.add("ndb_join_pushdown");
        mysqlBothVariables.add("ndb_log_bin");
        mysqlBothVariables.add("ndb_log_exclusive_reads");
        mysqlBothVariables.add("ndb_log_exclusive_reads");
        mysqlBothVariables.add("ndb_row_checksum");
        mysqlBothVariables.add("ndb_use_copying_alter_table");
        mysqlBothVariables.add("ndb_use_exact_count");
        mysqlBothVariables.add("ndb_use_transactions");
        mysqlBothVariables.add("ndbinfo_max_bytes");
        mysqlBothVariables.add("ndbinfo_max_rows");
        mysqlBothVariables.add("ndbinfo_show_hidden");
        mysqlBothVariables.add("net_buffer_length");
        mysqlBothVariables.add("net_read_timeout");
        mysqlBothVariables.add("net_retry_count");
        mysqlBothVariables.add("net_write_timeout");
        mysqlBothVariables.add("new");
        mysqlBothVariables.add("old_alter_table");
        mysqlBothVariables.add("optimizer_prune_level");
        mysqlBothVariables.add("optimizer_search_depth");
        mysqlBothVariables.add("optimizer_switch");
        mysqlBothVariables.add("optimizer_trace");
        mysqlBothVariables.add("optimizer_trace_features");
        mysqlBothVariables.add("optimizer_trace_limit");
        mysqlBothVariables.add("optimizer_trace_max_mem_size");
        mysqlBothVariables.add("optimizer_trace_offset");
        mysqlBothVariables.add("parser_max_mem_size");
        mysqlBothVariables.add("preload_buffer_size");
        mysqlBothVariables.add("print_identified_with_as_hex");
        mysqlBothVariables.add("profiling");
        mysqlBothVariables.add("profiling_history_size");
        mysqlBothVariables.add("query_alloc_block_size");
        mysqlBothVariables.add("query_prealloc_size");
        mysqlBothVariables.add("range_alloc_block_size");
        mysqlBothVariables.add("range_optimizer_max_mem_size");
        mysqlBothVariables.add("polarx_max_connections");
        mysqlBothVariables.add("polarx_max_allowed_packet");
        mysqlBothVariables.add("polarx_connect_timeout");
        mysqlBothVariables.add("read_buffer_size");
        mysqlBothVariables.add("read_rnd_buffer_size");
        mysqlBothVariables.add("select_into_buffer_size");
        mysqlBothVariables.add("select_into_disk_sync");
        mysqlBothVariables.add("select_into_disk_sync_delay");
        mysqlBothVariables.add("session_track_gtids");
        mysqlBothVariables.add("session_track_schema");
        mysqlBothVariables.add("session_track_state_change");
        mysqlBothVariables.add("session_track_system_variables");
        mysqlBothVariables.add("session_track_transaction_info");
        mysqlBothVariables.add("show_create_table_verbosity");
        mysqlBothVariables.add("show_old_temporals");
        mysqlBothVariables.add("sort_buffer_size");
        mysqlBothVariables.add("sql_auto_is_null");
        mysqlBothVariables.add("sql_big_selects");
        mysqlBothVariables.add("sql_buffer_result");
        mysqlBothVariables.add("sql_log_off");
        mysqlBothVariables.add("sql_mode");
        mysqlBothVariables.add("sql_notes");
        mysqlBothVariables.add("sql_quote_show_create");
        mysqlBothVariables.add("sql_require_primary_key");
        mysqlBothVariables.add("sql_safe_updates");
        mysqlBothVariables.add("sql_select_limit");
        mysqlBothVariables.add("sql_warnings");
        mysqlBothVariables.add("terminology_use_previous");
        mysqlBothVariables.add("thread_pool_high_priority_connection");
        mysqlBothVariables.add("time_zone");
        mysqlBothVariables.add("tmp_table_size");
        mysqlBothVariables.add("transaction_alloc_block_size");
        mysqlBothVariables.add("transaction_isolation");
        mysqlBothVariables.add("transaction_prealloc_size");
        mysqlBothVariables.add("transaction_read_only");
        mysqlBothVariables.add("transaction_write_set_extraction");
        mysqlBothVariables.add("unique_checks");
        mysqlBothVariables.add("updatable_views_with_limit");
        mysqlBothVariables.add("version_tokens_session");
        mysqlBothVariables.add("version_tokens_session_number");
        mysqlBothVariables.add("wait_timeout");
        mysqlBothVariables.add("windowing_use_high_precision");
        mysqlGlobalVariables.add("activate_all_roles_on_login");
        mysqlGlobalVariables.add("admin_address");
        mysqlGlobalVariables.add("innodb_purge_history");
        mysqlGlobalVariables.add("replica_read_timeout");
        mysqlGlobalVariables.add("loose_innodb_prepare_wait_timeout");
        mysqlGlobalVariables.add("innodb_prepare_wait_timeout");
        mysqlGlobalVariables.add("admin_port");
        mysqlGlobalVariables.add("admin_ssl_ca");
        mysqlGlobalVariables.add("admin_ssl_capath");
        mysqlGlobalVariables.add("admin_ssl_cert");
        mysqlGlobalVariables.add("admin_ssl_cipher");
        mysqlGlobalVariables.add("admin_ssl_crl");
        mysqlGlobalVariables.add("admin_ssl_crlpath");
        mysqlGlobalVariables.add("admin_ssl_key");
        mysqlGlobalVariables.add("admin_tls_ciphersuites");
        mysqlGlobalVariables.add("admin_tls_version");
        mysqlGlobalVariables.add("audit_log_buffer_size");
        mysqlGlobalVariables.add("audit_log_compression");
        mysqlGlobalVariables.add("audit_log_connection_policy");
        mysqlGlobalVariables.add("audit_log_encryption");
        mysqlGlobalVariables.add("audit_log_exclude_accounts");
        mysqlGlobalVariables.add("audit_log_file");
        mysqlGlobalVariables.add("audit_log_flush");
        mysqlGlobalVariables.add("audit_log_format");
        mysqlGlobalVariables.add("audit_log_format_unix_timestamp");
        mysqlGlobalVariables.add("audit_log_include_accounts");
        mysqlGlobalVariables.add("audit_log_max_size");
        mysqlGlobalVariables.add("audit_log_password_history_keep_days");
        mysqlGlobalVariables.add("audit_log_policy");
        mysqlGlobalVariables.add("audit_log_prune_seconds");
        mysqlGlobalVariables.add("audit_log_rotate_on_size");
        mysqlGlobalVariables.add("audit_log_statement_policy");
        mysqlGlobalVariables.add("audit_log_strategy");
        mysqlGlobalVariables.add("authentication_kerberos_service_key_tab");
        mysqlGlobalVariables.add("authentication_kerberos_service_principal");
        mysqlGlobalVariables.add("authentication_ldap_sasl_auth_method_name");
        mysqlGlobalVariables.add("authentication_ldap_sasl_bind_base_dn");
        mysqlGlobalVariables.add("authentication_ldap_sasl_bind_root_dn");
        mysqlGlobalVariables.add("authentication_ldap_sasl_bind_root_pwd");
        mysqlGlobalVariables.add("authentication_ldap_sasl_ca_path");
        mysqlGlobalVariables.add("authentication_ldap_sasl_group_search_attr");
        mysqlGlobalVariables.add("authentication_ldap_sasl_group_search_filter");
        mysqlGlobalVariables.add("authentication_ldap_sasl_init_pool_size");
        mysqlGlobalVariables.add("authentication_ldap_sasl_log_status");
        mysqlGlobalVariables.add("authentication_ldap_sasl_max_pool_size");
        mysqlGlobalVariables.add("authentication_ldap_sasl_referral");
        mysqlGlobalVariables.add("authentication_ldap_sasl_server_host");
        mysqlGlobalVariables.add("authentication_ldap_sasl_server_port");
        mysqlGlobalVariables.add("authentication_ldap_sasl_tls");
        mysqlGlobalVariables.add("authentication_ldap_sasl_user_search_attr");
        mysqlGlobalVariables.add("authentication_ldap_simple_auth_method_name");
        mysqlGlobalVariables.add("authentication_ldap_simple_bind_base_dn");
        mysqlGlobalVariables.add("authentication_ldap_simple_bind_root_dn");
        mysqlGlobalVariables.add("authentication_ldap_simple_bind_root_pwd");
        mysqlGlobalVariables.add("authentication_ldap_simple_ca_path");
        mysqlGlobalVariables.add("authentication_ldap_simple_group_search_attr");
        mysqlGlobalVariables.add("authentication_ldap_simple_group_search_filter");
        mysqlGlobalVariables.add("authentication_ldap_simple_init_pool_size");
        mysqlGlobalVariables.add("authentication_ldap_simple_log_status");
        mysqlGlobalVariables.add("authentication_ldap_simple_max_pool_size");
        mysqlGlobalVariables.add("authentication_ldap_simple_referral");
        mysqlGlobalVariables.add("authentication_ldap_simple_server_host");
        mysqlGlobalVariables.add("authentication_ldap_simple_server_port");
        mysqlGlobalVariables.add("authentication_ldap_simple_tls");
        mysqlGlobalVariables.add("authentication_ldap_simple_user_search_attr");
        mysqlGlobalVariables.add("authentication_windows_log_level");
        mysqlGlobalVariables.add("authentication_windows_use_principal_name");
        mysqlGlobalVariables.add("auto_generate_certs");
        mysqlGlobalVariables.add("automatic_sp_privileges");
        mysqlGlobalVariables.add("avoid_temporal_upgrade");
        mysqlGlobalVariables.add("back_log");
        mysqlGlobalVariables.add("basedir");
        mysqlGlobalVariables.add("bind_address");
        mysqlGlobalVariables.add("binlog_cache_size");
        mysqlGlobalVariables.add("binlog_checksum");
        mysqlGlobalVariables.add("binlog_encryption");
        mysqlGlobalVariables.add("binlog_error_action");
        mysqlGlobalVariables.add("binlog_expire_logs_seconds");
        mysqlGlobalVariables.add("binlog_group_commit_sync_delay");
        mysqlGlobalVariables.add("binlog_group_commit_sync_no_delay_count");
        mysqlGlobalVariables.add("binlog_gtid_simple_recovery");
        mysqlGlobalVariables.add("binlog_max_flush_queue_time");
        mysqlGlobalVariables.add("binlog_order_commits");
        mysqlGlobalVariables.add("binlog_rotate_encryption_master_key_at_startup");
        mysqlGlobalVariables.add("binlog_row_event_max_size");
        mysqlGlobalVariables.add("binlog_row_metadata");
        mysqlGlobalVariables.add("binlog_stmt_cache_size");
        mysqlGlobalVariables.add("binlog_transaction_dependency_history_size");
        mysqlGlobalVariables.add("binlog_transaction_dependency_tracking");
        mysqlGlobalVariables.add("caching_sha2_password_auto_generate_rsa_keys");
        mysqlGlobalVariables.add("caching_sha2_password_digest_rounds");
        mysqlGlobalVariables.add("caching_sha2_password_private_key_path");
        mysqlGlobalVariables.add("caching_sha2_password_public_key_path");
        mysqlGlobalVariables.add("character_set_system");
        mysqlGlobalVariables.add("character_sets_dir");
        mysqlGlobalVariables.add("check_proxy_users");
        mysqlGlobalVariables.add("clone_autotune_concurrency");
        mysqlGlobalVariables.add("clone_buffer_size");
        mysqlGlobalVariables.add("clone_ddl_timeout");
        mysqlGlobalVariables.add("clone_donor_timeout_after_network_failure");
        mysqlGlobalVariables.add("clone_enable_compression");
        mysqlGlobalVariables.add("clone_max_concurrency");
        mysqlGlobalVariables.add("clone_max_data_bandwidth");
        mysqlGlobalVariables.add("clone_max_network_bandwidth");
        mysqlGlobalVariables.add("clone_ssl_ca");
        mysqlGlobalVariables.add("clone_ssl_cert");
        mysqlGlobalVariables.add("clone_ssl_key");
        mysqlGlobalVariables.add("clone_valid_donor_list");
        mysqlGlobalVariables.add("concurrent_insert");
        mysqlGlobalVariables.add("connect_timeout");
        mysqlGlobalVariables.add("connection_control_failed_connections_threshold");
        mysqlGlobalVariables.add("connection_control_max_connection_delay");
        mysqlGlobalVariables.add("connection_control_min_connection_delay");
        mysqlGlobalVariables.add("core_file");
        mysqlGlobalVariables.add("create_admin_listener_thread");
        mysqlGlobalVariables.add("daemon_memcached_enable_binlog");
        mysqlGlobalVariables.add("daemon_memcached_engine_lib_name");
        mysqlGlobalVariables.add("daemon_memcached_engine_lib_path");
        mysqlGlobalVariables.add("daemon_memcached_option");
        mysqlGlobalVariables.add("daemon_memcached_r_batch_size");
        mysqlGlobalVariables.add("daemon_memcached_w_batch_size");
        mysqlGlobalVariables.add("datadir");
        mysqlGlobalVariables.add("default_authentication_plugin");
        mysqlGlobalVariables.add("default_password_lifetime");
        mysqlGlobalVariables.add("delay_key_write");
        mysqlGlobalVariables.add("delayed_insert_limit");
        mysqlGlobalVariables.add("delayed_insert_timeout");
        mysqlGlobalVariables.add("delayed_queue_size");
        mysqlGlobalVariables.add("disabled_storage_engines");
        mysqlGlobalVariables.add("disconnect_on_expired_password");
        mysqlGlobalVariables.add("dragnet.log_error_filter_rules");
        mysqlGlobalVariables.add("enforce_gtid_consistency");
        mysqlGlobalVariables.add("event_scheduler");
        mysqlGlobalVariables.add("expire_logs_days");
        mysqlGlobalVariables.add("flush");
        mysqlGlobalVariables.add("flush_time");
        mysqlGlobalVariables.add("ft_boolean_syntax");
        mysqlGlobalVariables.add("ft_max_word_len");
        mysqlGlobalVariables.add("ft_min_word_len");
        mysqlGlobalVariables.add("ft_query_expansion_limit");
        mysqlGlobalVariables.add("ft_stopword_file");
        mysqlGlobalVariables.add("general_log");
        mysqlGlobalVariables.add("general_log_file");
        mysqlGlobalVariables.add("group_replication_advertise_recovery_endpoints");
        mysqlGlobalVariables.add("group_replication_allow_local_lower_version_join");
        mysqlGlobalVariables.add("group_replication_auto_increment_increment");
        mysqlGlobalVariables.add("group_replication_autorejoin_tries");
        mysqlGlobalVariables.add("group_replication_bootstrap_group");
        mysqlGlobalVariables.add("group_replication_clone_threshold");
        mysqlGlobalVariables.add("group_replication_communication_debug_options");
        mysqlGlobalVariables.add("group_replication_communication_max_message_size");
        mysqlGlobalVariables.add("group_replication_components_stop_timeout");
        mysqlGlobalVariables.add("group_replication_compression_threshold");
        mysqlGlobalVariables.add("group_replication_enforce_update_everywhere_checks");
        mysqlGlobalVariables.add("group_replication_exit_state_action");
        mysqlGlobalVariables.add("group_replication_flow_control_applier_threshold");
        mysqlGlobalVariables.add("group_replication_flow_control_certifier_threshold");
        mysqlGlobalVariables.add("group_replication_flow_control_hold_percent");
        mysqlGlobalVariables.add("group_replication_flow_control_max_commit_quota");
        mysqlGlobalVariables.add("group_replication_flow_control_member_quota_percent");
        mysqlGlobalVariables.add("group_replication_flow_control_min_quota");
        mysqlGlobalVariables.add("group_replication_flow_control_min_recovery_quota");
        mysqlGlobalVariables.add("group_replication_flow_control_mode");
        mysqlGlobalVariables.add("group_replication_flow_control_period");
        mysqlGlobalVariables.add("group_replication_flow_control_release_percent");
        mysqlGlobalVariables.add("group_replication_force_members");
        mysqlGlobalVariables.add("group_replication_group_name");
        mysqlGlobalVariables.add("group_replication_group_seeds");
        mysqlGlobalVariables.add("group_replication_gtid_assignment_block_size");
        mysqlGlobalVariables.add("group_replication_ip_allowlist");
        mysqlGlobalVariables.add("group_replication_ip_whitelist");
        mysqlGlobalVariables.add("group_replication_local_address");
        mysqlGlobalVariables.add("group_replication_member_expel_timeout");
        mysqlGlobalVariables.add("group_replication_member_weight");
        mysqlGlobalVariables.add("group_replication_message_cache_size");
        mysqlGlobalVariables.add("group_replication_poll_spin_loops");
        mysqlGlobalVariables.add("group_replication_recovery_complete_at");
        mysqlGlobalVariables.add("group_replication_recovery_compression_algorithms");
        mysqlGlobalVariables.add("group_replication_recovery_get_public_key");
        mysqlGlobalVariables.add("group_replication_recovery_public_key_path");
        mysqlGlobalVariables.add("group_replication_recovery_reconnect_interval");
        mysqlGlobalVariables.add("group_replication_recovery_retry_count");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_ca");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_capath");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_cert");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_cipher");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_crl");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_crlpath");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_key");
        mysqlGlobalVariables.add("group_replication_recovery_ssl_verify_server_cert");
        mysqlGlobalVariables.add("group_replication_recovery_tls_ciphersuites");
        mysqlGlobalVariables.add("group_replication_recovery_tls_version");
        mysqlGlobalVariables.add("group_replication_recovery_use_ssl");
        mysqlGlobalVariables.add("group_replication_recovery_zstd_compression_level");
        mysqlGlobalVariables.add("group_replication_single_primary_mode");
        mysqlGlobalVariables.add("group_replication_ssl_mode");
        mysqlGlobalVariables.add("group_replication_start_on_boot");
        mysqlGlobalVariables.add("group_replication_tls_source");
        mysqlGlobalVariables.add("group_replication_transaction_size_limit");
        mysqlGlobalVariables.add("group_replication_unreachable_majority_timeout");
        mysqlGlobalVariables.add("group_replication_view_change_uuid");
        mysqlGlobalVariables.add("gtid_executed");
        mysqlGlobalVariables.add("gtid_executed_compression_period");
        mysqlGlobalVariables.add("gtid_mode");
        mysqlGlobalVariables.add("gtid_purged");
        mysqlGlobalVariables.add("have_compress");
        mysqlGlobalVariables.add("have_dynamic_loading");
        mysqlGlobalVariables.add("have_geometry");
        mysqlGlobalVariables.add("have_openssl");
        mysqlGlobalVariables.add("have_profiling");
        mysqlGlobalVariables.add("have_query_cache");
        mysqlGlobalVariables.add("have_rtree_keys");
        mysqlGlobalVariables.add("have_ssl");
        mysqlGlobalVariables.add("have_statement_timeout");
        mysqlGlobalVariables.add("have_symlink");
        mysqlGlobalVariables.add("host_cache_size");
        mysqlGlobalVariables.add("hostname");
        mysqlGlobalVariables.add("init_connect");
        mysqlGlobalVariables.add("init_file");
        mysqlGlobalVariables.add("init_replica");
        mysqlGlobalVariables.add("init_slave");
        mysqlGlobalVariables.add("innodb_adaptive_flushing");
        mysqlGlobalVariables.add("innodb_adaptive_flushing_lwm");
        mysqlGlobalVariables.add("innodb_adaptive_hash_index");
        mysqlGlobalVariables.add("innodb_adaptive_hash_index_parts");
        mysqlGlobalVariables.add("innodb_adaptive_max_sleep_delay");
        mysqlGlobalVariables.add("innodb_api_bk_commit_interval");
        mysqlGlobalVariables.add("innodb_api_disable_rowlock");
        mysqlGlobalVariables.add("innodb_api_enable_binlog");
        mysqlGlobalVariables.add("innodb_api_enable_mdl");
        mysqlGlobalVariables.add("innodb_api_trx_level");
        mysqlGlobalVariables.add("innodb_autoextend_increment");
        mysqlGlobalVariables.add("innodb_autoinc_lock_mode");
        mysqlGlobalVariables.add("innodb_background_drop_list_empty");
        mysqlGlobalVariables.add("innodb_buffer_pool_chunk_size");
        mysqlGlobalVariables.add("innodb_buffer_pool_debug");
        mysqlGlobalVariables.add("innodb_buffer_pool_dump_at_shutdown");
        mysqlGlobalVariables.add("innodb_buffer_pool_dump_now");
        mysqlGlobalVariables.add("innodb_buffer_pool_dump_pct");
        mysqlGlobalVariables.add("innodb_buffer_pool_filename");
        mysqlGlobalVariables.add("innodb_buffer_pool_in_core_file");
        mysqlGlobalVariables.add("innodb_buffer_pool_instances");
        mysqlGlobalVariables.add("innodb_buffer_pool_load_abort");
        mysqlGlobalVariables.add("innodb_buffer_pool_load_at_startup");
        mysqlGlobalVariables.add("innodb_buffer_pool_load_now");
        mysqlGlobalVariables.add("innodb_buffer_pool_size");
        mysqlGlobalVariables.add("innodb_change_buffer_max_size");
        mysqlGlobalVariables.add("innodb_change_buffering");
        mysqlGlobalVariables.add("innodb_change_buffering_debug");
        mysqlGlobalVariables.add("innodb_checkpoint_disabled");
        mysqlGlobalVariables.add("innodb_checksum_algorithm");
        mysqlGlobalVariables.add("innodb_cmp_per_index_enabled");
        mysqlGlobalVariables.add("innodb_commit_concurrency");
        mysqlGlobalVariables.add("innodb_compress_debug");
        mysqlGlobalVariables.add("innodb_compression_failure_threshold_pct");
        mysqlGlobalVariables.add("innodb_compression_level");
        mysqlGlobalVariables.add("innodb_compression_pad_pct_max");
        mysqlGlobalVariables.add("innodb_concurrency_tickets");
        mysqlGlobalVariables.add("innodb_data_file_path");
        mysqlGlobalVariables.add("innodb_data_file_purge");
        mysqlGlobalVariables.add("innodb_data_file_purge_interval");
        mysqlGlobalVariables.add("innodb_data_file_purge_max_size");
        mysqlGlobalVariables.add("innodb_data_home_dir");
        mysqlGlobalVariables.add("innodb_ddl_log_crash_reset_debug");
        mysqlGlobalVariables.add("innodb_deadlock_detect");
        mysqlGlobalVariables.add("innodb_dedicated_server");
        mysqlGlobalVariables.add("innodb_default_row_format");
        mysqlGlobalVariables.add("innodb_directories");
        mysqlGlobalVariables.add("innodb_disable_sort_file_cache");
        mysqlGlobalVariables.add("innodb_doublewrite");
        mysqlGlobalVariables.add("innodb_doublewrite_batch_size");
        mysqlGlobalVariables.add("innodb_doublewrite_dir");
        mysqlGlobalVariables.add("innodb_doublewrite_files");
        mysqlGlobalVariables.add("innodb_doublewrite_pages");
        mysqlGlobalVariables.add("innodb_extend_and_initialize");
        mysqlGlobalVariables.add("innodb_fast_shutdown");
        mysqlGlobalVariables.add("innodb_fil_make_page_dirty_debug");
        mysqlGlobalVariables.add("innodb_file_per_table");
        mysqlGlobalVariables.add("innodb_fill_factor");
        mysqlGlobalVariables.add("innodb_flush_log_at_timeout");
        mysqlGlobalVariables.add("innodb_flush_log_at_trx_commit");
        mysqlGlobalVariables.add("innodb_flush_method");
        mysqlGlobalVariables.add("innodb_flush_neighbors");
        mysqlGlobalVariables.add("innodb_flush_sync");
        mysqlGlobalVariables.add("innodb_flushing_avg_loops");
        mysqlGlobalVariables.add("innodb_force_load_corrupted");
        mysqlGlobalVariables.add("innodb_force_recovery");
        mysqlGlobalVariables.add("innodb_fsync_threshold");
        mysqlGlobalVariables.add("innodb_ft_aux_table");
        mysqlGlobalVariables.add("innodb_ft_cache_size");
        mysqlGlobalVariables.add("innodb_ft_enable_diag_print");
        mysqlGlobalVariables.add("innodb_ft_max_token_size");
        mysqlGlobalVariables.add("innodb_ft_min_token_size");
        mysqlGlobalVariables.add("innodb_ft_num_word_optimize");
        mysqlGlobalVariables.add("innodb_ft_result_cache_limit");
        mysqlGlobalVariables.add("innodb_ft_server_stopword_table");
        mysqlGlobalVariables.add("innodb_ft_sort_pll_degree");
        mysqlGlobalVariables.add("innodb_ft_total_cache_size");
        mysqlGlobalVariables.add("innodb_idle_flush_pct");
        mysqlGlobalVariables.add("innodb_io_capacity");
        mysqlGlobalVariables.add("innodb_io_capacity_max");
        mysqlGlobalVariables.add("innodb_limit_optimistic_insert_debug");
        mysqlGlobalVariables.add("innodb_log_buffer_size");
        mysqlGlobalVariables.add("innodb_log_checkpoint_fuzzy_now");
        mysqlGlobalVariables.add("innodb_log_checkpoint_now");
        mysqlGlobalVariables.add("innodb_log_checksums");
        mysqlGlobalVariables.add("innodb_log_compressed_pages");
        mysqlGlobalVariables.add("innodb_log_file_size");
        mysqlGlobalVariables.add("innodb_log_files_in_group");
        mysqlGlobalVariables.add("innodb_log_group_home_dir");
        mysqlGlobalVariables.add("innodb_log_spin_cpu_abs_lwm");
        mysqlGlobalVariables.add("innodb_log_spin_cpu_pct_hwm");
        mysqlGlobalVariables.add("innodb_log_wait_for_flush_spin_hwm");
        mysqlGlobalVariables.add("innodb_log_write_ahead_size");
        mysqlGlobalVariables.add("innodb_log_writer_threads");
        mysqlGlobalVariables.add("innodb_lru_scan_depth");
        mysqlGlobalVariables.add("innodb_max_dirty_pages_pct");
        mysqlGlobalVariables.add("innodb_max_dirty_pages_pct_lwm");
        mysqlGlobalVariables.add("innodb_max_purge_lag");
        mysqlGlobalVariables.add("innodb_max_purge_lag_delay");
        mysqlGlobalVariables.add("innodb_max_undo_log_size");
        mysqlGlobalVariables.add("innodb_merge_threshold_set_all_debug");
        mysqlGlobalVariables.add("innodb_monitor_disable");
        mysqlGlobalVariables.add("innodb_monitor_enable");
        mysqlGlobalVariables.add("innodb_monitor_reset");
        mysqlGlobalVariables.add("innodb_monitor_reset_all");
        mysqlGlobalVariables.add("innodb_numa_interleave");
        mysqlGlobalVariables.add("innodb_old_blocks_pct");
        mysqlGlobalVariables.add("innodb_old_blocks_time");
        mysqlGlobalVariables.add("innodb_online_alter_log_max_size");
        mysqlGlobalVariables.add("innodb_open_files");
        mysqlGlobalVariables.add("innodb_optimize_fulltext_only");
        mysqlGlobalVariables.add("innodb_page_cleaners");
        mysqlGlobalVariables.add("innodb_page_size");
        mysqlGlobalVariables.add("innodb_print_all_deadlocks");
        mysqlGlobalVariables.add("innodb_print_ddl_logs");
        mysqlGlobalVariables.add("innodb_purge_batch_size");
        mysqlGlobalVariables.add("innodb_purge_rseg_truncate_frequency");
        mysqlGlobalVariables.add("innodb_purge_threads");
        mysqlGlobalVariables.add("innodb_random_read_ahead");
        mysqlGlobalVariables.add("innodb_read_ahead_threshold");
        mysqlGlobalVariables.add("innodb_read_io_threads");
        mysqlGlobalVariables.add("innodb_read_only");
        mysqlGlobalVariables.add("innodb_redo_log_archive_dirs");
        mysqlGlobalVariables.add("innodb_redo_log_encrypt");
        mysqlGlobalVariables.add("innodb_replication_delay");
        mysqlGlobalVariables.add("innodb_rollback_on_timeout");
        mysqlGlobalVariables.add("innodb_rollback_segments");
        mysqlGlobalVariables.add("innodb_saved_page_number_debug");
        mysqlGlobalVariables.add("innodb_segment_reserve_factor");
        mysqlGlobalVariables.add("innodb_sort_buffer_size");
        mysqlGlobalVariables.add("innodb_spin_wait_delay");
        mysqlGlobalVariables.add("innodb_spin_wait_pause_multiplier");
        mysqlGlobalVariables.add("innodb_stats_auto_recalc");
        mysqlGlobalVariables.add("innodb_stats_include_delete_marked");
        mysqlGlobalVariables.add("innodb_stats_method");
        mysqlGlobalVariables.add("innodb_stats_on_metadata");
        mysqlGlobalVariables.add("innodb_stats_persistent");
        mysqlGlobalVariables.add("innodb_stats_persistent_sample_pages");
        mysqlGlobalVariables.add("innodb_stats_transient_sample_pages");
        mysqlGlobalVariables.add("innodb_status_output");
        mysqlGlobalVariables.add("innodb_status_output_locks");
        mysqlGlobalVariables.add("innodb_support_instant_add_column");
        mysqlGlobalVariables.add("innodb_sync_array_size");
        mysqlGlobalVariables.add("innodb_sync_debug");
        mysqlGlobalVariables.add("innodb_sync_spin_loops");
        mysqlGlobalVariables.add("innodb_temp_data_file_path");
        mysqlGlobalVariables.add("innodb_temp_tablespaces_dir");
        mysqlGlobalVariables.add("innodb_thread_concurrency");
        mysqlGlobalVariables.add("innodb_thread_sleep_delay");
        mysqlGlobalVariables.add("innodb_trx_purge_view_update_only_debug");
        mysqlGlobalVariables.add("innodb_trx_rseg_n_slots_debug");
        mysqlGlobalVariables.add("innodb_undo_directory");
        mysqlGlobalVariables.add("innodb_undo_log_encrypt");
        mysqlGlobalVariables.add("innodb_undo_log_truncate");
        mysqlGlobalVariables.add("innodb_undo_tablespaces");
        mysqlGlobalVariables.add("innodb_use_fdatasync");
        mysqlGlobalVariables.add("innodb_use_native_aio");
        mysqlGlobalVariables.add("innodb_validate_tablespace_paths");
        mysqlGlobalVariables.add("innodb_version");
        mysqlGlobalVariables.add("innodb_write_io_threads");
        mysqlGlobalVariables.add("internal_tmp_disk_storage_engine");
        mysqlGlobalVariables.add("key_buffer_size");
        mysqlGlobalVariables.add("key_cache_age_threshold");
        mysqlGlobalVariables.add("key_cache_block_size");
        mysqlGlobalVariables.add("key_cache_division_limit");
        mysqlGlobalVariables.add("keyring_aws_cmk_id");
        mysqlGlobalVariables.add("keyring_aws_conf_file");
        mysqlGlobalVariables.add("keyring_aws_data_file");
        mysqlGlobalVariables.add("keyring_aws_region");
        mysqlGlobalVariables.add("keyring_encrypted_file_data");
        mysqlGlobalVariables.add("keyring_encrypted_file_password");
        mysqlGlobalVariables.add("keyring_file_data");
        mysqlGlobalVariables.add("keyring_hashicorp_auth_path");
        mysqlGlobalVariables.add("keyring_hashicorp_ca_path");
        mysqlGlobalVariables.add("keyring_hashicorp_caching");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_auth_path");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_ca_path");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_caching");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_role_id");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_server_url");
        mysqlGlobalVariables.add("keyring_hashicorp_commit_store_path");
        mysqlGlobalVariables.add("keyring_hashicorp_role_id");
        mysqlGlobalVariables.add("keyring_hashicorp_secret_id");
        mysqlGlobalVariables.add("keyring_hashicorp_server_url");
        mysqlGlobalVariables.add("keyring_hashicorp_store_path");
        mysqlGlobalVariables.add("keyring_oci_ca_certificate");
        mysqlGlobalVariables.add("keyring_oci_compartment");
        mysqlGlobalVariables.add("keyring_oci_encryption_endpoint");
        mysqlGlobalVariables.add("keyring_oci_key_file");
        mysqlGlobalVariables.add("keyring_oci_key_fingerprint");
        mysqlGlobalVariables.add("keyring_oci_management_endpoint");
        mysqlGlobalVariables.add("keyring_oci_master_key");
        mysqlGlobalVariables.add("keyring_oci_secrets_endpoint");
        mysqlGlobalVariables.add("keyring_oci_tenancy");
        mysqlGlobalVariables.add("keyring_oci_user");
        mysqlGlobalVariables.add("keyring_oci_vaults_endpoint");
        mysqlGlobalVariables.add("keyring_oci_virtual_vault");
        mysqlGlobalVariables.add("keyring_okv_conf_dir");
        mysqlGlobalVariables.add("keyring_operations");
        mysqlGlobalVariables.add("language");
        mysqlGlobalVariables.add("large_files_support");
        mysqlGlobalVariables.add("large_page_size");
        mysqlGlobalVariables.add("large_pages");
        mysqlGlobalVariables.add("lc_messages_dir");
        mysqlGlobalVariables.add("license");
        mysqlGlobalVariables.add("local_infile");
        mysqlGlobalVariables.add("lock_order");
        mysqlGlobalVariables.add("lock_order_debug_loop");
        mysqlGlobalVariables.add("lock_order_debug_missing_arc");
        mysqlGlobalVariables.add("lock_order_debug_missing_key");
        mysqlGlobalVariables.add("lock_order_debug_missing_unlock");
        mysqlGlobalVariables.add("lock_order_dependencies");
        mysqlGlobalVariables.add("lock_order_extra_dependencies");
        mysqlGlobalVariables.add("lock_order_output_directory");
        mysqlGlobalVariables.add("lock_order_print_txt");
        mysqlGlobalVariables.add("lock_order_trace_loop");
        mysqlGlobalVariables.add("lock_order_trace_missing_arc");
        mysqlGlobalVariables.add("lock_order_trace_missing_key");
        mysqlGlobalVariables.add("lock_order_trace_missing_unlock");
        mysqlGlobalVariables.add("locked_in_memory");
        mysqlGlobalVariables.add("log_bin");
        mysqlGlobalVariables.add("log_bin_basename");
        mysqlGlobalVariables.add("log_bin_index");
        mysqlGlobalVariables.add("log_bin_trust_function_creators");
        mysqlGlobalVariables.add("log_bin_use_v1_row_events");
        mysqlGlobalVariables.add("log_error");
        mysqlGlobalVariables.add("log_error_services");
        mysqlGlobalVariables.add("log_error_suppression_list");
        mysqlGlobalVariables.add("log_error_verbosity");
        mysqlGlobalVariables.add("log_output");
        mysqlGlobalVariables.add("log_queries_not_using_indexes");
        mysqlGlobalVariables.add("log_raw");
        mysqlGlobalVariables.add("log_replica_updates");
        mysqlGlobalVariables.add("log_slave_updates");
        mysqlGlobalVariables.add("log_slow_admin_statements");
        mysqlGlobalVariables.add("log_slow_extra");
        mysqlGlobalVariables.add("log_slow_replica_statements");
        mysqlGlobalVariables.add("log_slow_slave_statements");
        mysqlGlobalVariables.add("log_statements_unsafe_for_binlog");
        mysqlGlobalVariables.add("log_syslog");
        mysqlGlobalVariables.add("log_syslog_facility");
        mysqlGlobalVariables.add("log_syslog_include_pid");
        mysqlGlobalVariables.add("log_syslog_tag");
        mysqlGlobalVariables.add("log_throttle_queries_not_using_indexes");
        mysqlGlobalVariables.add("log_timestamps");
        mysqlGlobalVariables.add("lower_case_file_system");
        mysqlGlobalVariables.add("lower_case_table_names");
        mysqlGlobalVariables.add("mandatory_roles");
        mysqlGlobalVariables.add("master_info_repository");
        mysqlGlobalVariables.add("master_verify_checksum");
        mysqlGlobalVariables.add("max_binlog_cache_size");
        mysqlGlobalVariables.add("max_binlog_size");
        mysqlGlobalVariables.add("max_binlog_stmt_cache_size");
        mysqlGlobalVariables.add("max_connect_errors");
        mysqlGlobalVariables.add("max_connections");
        mysqlGlobalVariables.add("max_digest_length");
        mysqlGlobalVariables.add("max_prepared_stmt_count");
        mysqlGlobalVariables.add("max_relay_log_size");
        mysqlGlobalVariables.add("max_write_lock_count");
        mysqlGlobalVariables.add("mecab_rc_file");
        mysqlGlobalVariables.add("metadata_locks_cache_size");
        mysqlGlobalVariables.add("metadata_locks_hash_instances");
        mysqlGlobalVariables.add("myisam_data_pointer_size");
        mysqlGlobalVariables.add("myisam_max_sort_file_size");
        mysqlGlobalVariables.add("myisam_mmap_size");
        mysqlGlobalVariables.add("myisam_recover_options");
        mysqlGlobalVariables.add("myisam_use_mmap");
        mysqlGlobalVariables.add("mysql_firewall_mode");
        mysqlGlobalVariables.add("mysql_firewall_trace");
        mysqlGlobalVariables.add("mysql_native_password_proxy_users");
        mysqlGlobalVariables.add("mysqlx_bind_address");
        mysqlGlobalVariables.add("mysqlx_compression_algorithms");
        mysqlGlobalVariables.add("mysqlx_connect_timeout");
        mysqlGlobalVariables.add("mysqlx_deflate_default_compression_level");
        mysqlGlobalVariables.add("mysqlx_deflate_max_client_compression_level");
        mysqlGlobalVariables.add("mysqlx_document_id_unique_prefix");
        mysqlGlobalVariables.add("mysqlx_enable_hello_notice");
        mysqlGlobalVariables.add("mysqlx_idle_worker_thread_timeout");
        mysqlGlobalVariables.add("mysqlx_interactive_timeout");
        mysqlGlobalVariables.add("mysqlx_lz4_default_compression_level");
        mysqlGlobalVariables.add("mysqlx_lz4_max_client_compression_level");
        mysqlGlobalVariables.add("mysqlx_max_allowed_packet");
        mysqlGlobalVariables.add("mysqlx_max_connections");
        mysqlGlobalVariables.add("mysqlx_min_worker_threads");
        mysqlGlobalVariables.add("mysqlx_port");
        mysqlGlobalVariables.add("mysqlx_port_open_timeout");
        mysqlGlobalVariables.add("mysqlx_socket");
        mysqlGlobalVariables.add("mysqlx_ssl_ca");
        mysqlGlobalVariables.add("mysqlx_ssl_capath");
        mysqlGlobalVariables.add("mysqlx_ssl_cert");
        mysqlGlobalVariables.add("mysqlx_ssl_cipher");
        mysqlGlobalVariables.add("mysqlx_ssl_crl");
        mysqlGlobalVariables.add("mysqlx_ssl_crlpath");
        mysqlGlobalVariables.add("mysqlx_ssl_key");
        mysqlGlobalVariables.add("mysqlx_zstd_default_compression_level");
        mysqlGlobalVariables.add("mysqlx_zstd_max_client_compression_level");
        mysqlGlobalVariables.add("named_pipe");
        mysqlGlobalVariables.add("named_pipe_full_access_group");
        mysqlGlobalVariables.add("ndb_batch_size");
        mysqlGlobalVariables.add("ndb_cache_check_time");
        mysqlGlobalVariables.add("ndb_clear_apply_status");
        mysqlGlobalVariables.add("ndb_cluster_connection_pool");
        mysqlGlobalVariables.add("ndb_cluster_connection_pool_nodeids");
        mysqlGlobalVariables.add("Ndb_conflict_last_conflict_epoch");
        mysqlGlobalVariables.add("ndb_conflict_role");
        mysqlGlobalVariables.add("ndb_data_node_neighbour");
        mysqlGlobalVariables.add("ndb_default_column_format");
        mysqlGlobalVariables.add("ndb_default_column_format");
        mysqlGlobalVariables.add("ndb_distribution");
        mysqlGlobalVariables.add("ndb_distribution");
        mysqlGlobalVariables.add("ndb_eventbuffer_free_percent");
        mysqlGlobalVariables.add("ndb_eventbuffer_max_alloc");
        mysqlGlobalVariables.add("ndb_extra_logging");
        mysqlGlobalVariables.add("ndb_log_apply_status");
        mysqlGlobalVariables.add("ndb_log_apply_status");
        mysqlGlobalVariables.add("ndb_log_binlog_index");
        mysqlGlobalVariables.add("ndb_log_empty_epochs");
        mysqlGlobalVariables.add("ndb_log_empty_epochs");
        mysqlGlobalVariables.add("ndb_log_empty_update");
        mysqlGlobalVariables.add("ndb_log_empty_update");
        mysqlGlobalVariables.add("ndb_log_fail_terminate");
        mysqlGlobalVariables.add("ndb_log_orig");
        mysqlGlobalVariables.add("ndb_log_orig");
        mysqlGlobalVariables.add("ndb_log_transaction_id");
        mysqlGlobalVariables.add("ndb_log_transaction_id");
        mysqlGlobalVariables.add("ndb_log_update_as_write");
        mysqlGlobalVariables.add("ndb_log_update_minimal");
        mysqlGlobalVariables.add("ndb_log_updated_only");
        mysqlGlobalVariables.add("ndb_metadata_check");
        mysqlGlobalVariables.add("ndb_metadata_check_interval");
        mysqlGlobalVariables.add("ndb_metadata_sync");
        mysqlGlobalVariables.add("ndb_optimization_delay");
        mysqlGlobalVariables.add("ndb_optimized_node_selection");
        mysqlGlobalVariables.add("ndb_read_backup");
        mysqlGlobalVariables.add("ndb_recv_thread_activation_threshold");
        mysqlGlobalVariables.add("ndb_recv_thread_cpu_mask");
        mysqlGlobalVariables.add("Ndb_replica_max_replicated_epoch");
        mysqlGlobalVariables.add("ndb_report_thresh_binlog_epoch_slip");
        mysqlGlobalVariables.add("ndb_report_thresh_binlog_mem_usage");
        mysqlGlobalVariables.add("ndb_schema_dist_lock_wait_timeout");
        mysqlGlobalVariables.add("ndb_schema_dist_timeout");
        mysqlGlobalVariables.add("ndb_schema_dist_timeout");
        mysqlGlobalVariables.add("ndb_schema_dist_upgrade_allowed");
        mysqlGlobalVariables.add("ndb_show_foreign_key_mock_tables");
        mysqlGlobalVariables.add("ndb_slave_conflict_role");
        mysqlGlobalVariables.add("Ndb_slave_max_replicated_epoch");
        mysqlGlobalVariables.add("Ndb_system_name");
        mysqlGlobalVariables.add("ndb_version");
        mysqlGlobalVariables.add("ndb_version_string");
        mysqlGlobalVariables.add("ndb_wait_connected");
        mysqlGlobalVariables.add("ndb_wait_setup");
        mysqlGlobalVariables.add("ndbinfo_database");
        mysqlGlobalVariables.add("ndbinfo_offline");
        mysqlGlobalVariables.add("ndbinfo_table_prefix");
        mysqlGlobalVariables.add("ndbinfo_version");
        mysqlGlobalVariables.add("ngram_token_size");
        mysqlGlobalVariables.add("offline_mode");
        mysqlGlobalVariables.add("old");
        mysqlGlobalVariables.add("open_files_limit");
        mysqlGlobalVariables.add("partial_revokes");
        mysqlGlobalVariables.add("password_history");
        mysqlGlobalVariables.add("password_require_current");
        mysqlGlobalVariables.add("password_reuse_interval");
        mysqlGlobalVariables.add("performance_schema");
        mysqlGlobalVariables.add("performance_schema_accounts_size");
        mysqlGlobalVariables.add("performance_schema_digests_size");
        mysqlGlobalVariables.add("performance_schema_error_size");
        mysqlGlobalVariables.add("performance_schema_events_stages_history_long_size");
        mysqlGlobalVariables.add("performance_schema_events_stages_history_size");
        mysqlGlobalVariables.add("performance_schema_events_statements_history_long_size");
        mysqlGlobalVariables.add("performance_schema_events_statements_history_size");
        mysqlGlobalVariables.add("performance_schema_events_transactions_history_long_size");
        mysqlGlobalVariables.add("performance_schema_events_transactions_history_size");
        mysqlGlobalVariables.add("performance_schema_events_waits_history_long_size");
        mysqlGlobalVariables.add("performance_schema_events_waits_history_size");
        mysqlGlobalVariables.add("performance_schema_hosts_size");
        mysqlGlobalVariables.add("performance_schema_max_cond_classes");
        mysqlGlobalVariables.add("performance_schema_max_cond_instances");
        mysqlGlobalVariables.add("performance_schema_max_digest_length");
        mysqlGlobalVariables.add("performance_schema_max_digest_sample_age");
        mysqlGlobalVariables.add("performance_schema_max_file_classes");
        mysqlGlobalVariables.add("performance_schema_max_file_handles");
        mysqlGlobalVariables.add("performance_schema_max_file_instances");
        mysqlGlobalVariables.add("performance_schema_max_index_stat");
        mysqlGlobalVariables.add("performance_schema_max_memory_classes");
        mysqlGlobalVariables.add("performance_schema_max_metadata_locks");
        mysqlGlobalVariables.add("performance_schema_max_mutex_classes");
        mysqlGlobalVariables.add("performance_schema_max_mutex_instances");
        mysqlGlobalVariables.add("performance_schema_max_prepared_statements_instances");
        mysqlGlobalVariables.add("performance_schema_max_program_instances");
        mysqlGlobalVariables.add("performance_schema_max_rwlock_classes");
        mysqlGlobalVariables.add("performance_schema_max_rwlock_instances");
        mysqlGlobalVariables.add("performance_schema_max_socket_classes");
        mysqlGlobalVariables.add("performance_schema_max_socket_instances");
        mysqlGlobalVariables.add("performance_schema_max_sql_text_length");
        mysqlGlobalVariables.add("performance_schema_max_stage_classes");
        mysqlGlobalVariables.add("performance_schema_max_statement_classes");
        mysqlGlobalVariables.add("performance_schema_max_statement_stack");
        mysqlGlobalVariables.add("performance_schema_max_table_handles");
        mysqlGlobalVariables.add("performance_schema_max_table_instances");
        mysqlGlobalVariables.add("performance_schema_max_table_lock_stat");
        mysqlGlobalVariables.add("performance_schema_max_thread_classes");
        mysqlGlobalVariables.add("performance_schema_max_thread_instances");
        mysqlGlobalVariables.add("performance_schema_session_connect_attrs_size");
        mysqlGlobalVariables.add("performance_schema_setup_actors_size");
        mysqlGlobalVariables.add("performance_schema_setup_objects_size");
        mysqlGlobalVariables.add("performance_schema_show_processlist");
        mysqlGlobalVariables.add("performance_schema_users_size");
        mysqlGlobalVariables.add("persist_only_admin_x509_subject");
        mysqlGlobalVariables.add("persisted_globals_load");
        mysqlGlobalVariables.add("pid_file");
        mysqlGlobalVariables.add("plugin_dir");
        mysqlGlobalVariables.add("plugin_load");
        mysqlGlobalVariables.add("plugin_load_add");
        mysqlGlobalVariables.add("port");
        mysqlGlobalVariables.add("protocol_compression_algorithms");
        mysqlGlobalVariables.add("protocol_version");
        mysqlGlobalVariables.add("read_only");
        mysqlGlobalVariables.add("regexp_stack_limit");
        mysqlGlobalVariables.add("regexp_time_limit");
        mysqlGlobalVariables.add("relay_log");
        mysqlGlobalVariables.add("relay_log_basename");
        mysqlGlobalVariables.add("relay_log_index");
        mysqlGlobalVariables.add("relay_log_info_file");
        mysqlGlobalVariables.add("relay_log_info_repository");
        mysqlGlobalVariables.add("relay_log_purge");
        mysqlGlobalVariables.add("relay_log_recovery");
        mysqlGlobalVariables.add("relay_log_space_limit");
        mysqlGlobalVariables.add("replica_allow_batching");
        mysqlGlobalVariables.add("replica_checkpoint_group");
        mysqlGlobalVariables.add("replica_checkpoint_period");
        mysqlGlobalVariables.add("replica_compressed_protocol");
        mysqlGlobalVariables.add("replica_exec_mode");
        mysqlGlobalVariables.add("replica_load_tmpdir");
        mysqlGlobalVariables.add("replica_max_allowed_packet");
        mysqlGlobalVariables.add("replica_net_timeout");
        mysqlGlobalVariables.add("replica_parallel_type");
        mysqlGlobalVariables.add("replica_parallel_workers");
        mysqlGlobalVariables.add("replica_pending_jobs_size_max");
        mysqlGlobalVariables.add("replica_preserve_commit_order");
        mysqlGlobalVariables.add("replica_skip_errors");
        mysqlGlobalVariables.add("replica_sql_verify_checksum");
        mysqlGlobalVariables.add("replica_transaction_retries");
        mysqlGlobalVariables.add("replica_type_conversions");
        mysqlGlobalVariables.add("replication_optimize_for_static_plugin_config");
        mysqlGlobalVariables.add("replication_sender_observe_commit_only");
        mysqlGlobalVariables.add("report_host");
        mysqlGlobalVariables.add("report_password");
        mysqlGlobalVariables.add("report_port");
        mysqlGlobalVariables.add("report_user");
        mysqlGlobalVariables.add("require_secure_transport");
        mysqlGlobalVariables.add("rewriter_enabled");
        mysqlGlobalVariables.add("rewriter_verbose");
        mysqlGlobalVariables.add("rpl_read_size");
        mysqlGlobalVariables.add("rpl_semi_sync_master_enabled");
        mysqlGlobalVariables.add("rpl_semi_sync_master_timeout");
        mysqlGlobalVariables.add("rpl_semi_sync_master_trace_level");
        mysqlGlobalVariables.add("rpl_semi_sync_master_wait_for_slave_count");
        mysqlGlobalVariables.add("rpl_semi_sync_master_wait_no_slave");
        mysqlGlobalVariables.add("rpl_semi_sync_master_wait_point");
        mysqlGlobalVariables.add("rpl_semi_sync_replica_enabled");
        mysqlGlobalVariables.add("rpl_semi_sync_replica_trace_level");
        mysqlGlobalVariables.add("rpl_semi_sync_slave_enabled");
        mysqlGlobalVariables.add("rpl_semi_sync_slave_trace_level");
        mysqlGlobalVariables.add("rpl_semi_sync_source_enabled");
        mysqlGlobalVariables.add("rpl_semi_sync_source_timeout");
        mysqlGlobalVariables.add("rpl_semi_sync_source_trace_level");
        mysqlGlobalVariables.add("rpl_semi_sync_source_wait_for_replica_count");
        mysqlGlobalVariables.add("rpl_semi_sync_source_wait_no_replica");
        mysqlGlobalVariables.add("rpl_semi_sync_source_wait_point");
        mysqlGlobalVariables.add("rpl_stop_replica_timeout");
        mysqlGlobalVariables.add("rpl_stop_slave_timeout");
        mysqlGlobalVariables.add("schema_definition_cache");
        mysqlGlobalVariables.add("secure_file_priv");
        mysqlGlobalVariables.add("server_id");
        mysqlGlobalVariables.add("server_id_bits");
        mysqlGlobalVariables.add("server_uuid");
        mysqlGlobalVariables.add("sha256_password_auto_generate_rsa_keys");
        mysqlGlobalVariables.add("sha256_password_private_key_path");
        mysqlGlobalVariables.add("sha256_password_proxy_users");
        mysqlGlobalVariables.add("sha256_password_public_key_path");
        mysqlGlobalVariables.add("shared_memory");
        mysqlGlobalVariables.add("shared_memory_base_name");
        mysqlGlobalVariables.add("skip_external_locking");
        mysqlGlobalVariables.add("skip_name_resolve");
        mysqlGlobalVariables.add("skip_networking");
        mysqlGlobalVariables.add("skip_replica_start");
        mysqlGlobalVariables.add("skip_show_database");
        mysqlGlobalVariables.add("skip_slave_start");
        mysqlGlobalVariables.add("slave_allow_batching");
        mysqlGlobalVariables.add("slave_checkpoint_group");
        mysqlGlobalVariables.add("slave_checkpoint_period");
        mysqlGlobalVariables.add("slave_compressed_protocol");
        mysqlGlobalVariables.add("slave_exec_mode");
        mysqlGlobalVariables.add("slave_load_tmpdir");
        mysqlGlobalVariables.add("slave_max_allowed_packet");
        mysqlGlobalVariables.add("slave_net_timeout");
        mysqlGlobalVariables.add("slave_parallel_type");
        mysqlGlobalVariables.add("slave_parallel_workers");
        mysqlGlobalVariables.add("slave_pending_jobs_size_max");
        mysqlGlobalVariables.add("slave_preserve_commit_order");
        mysqlGlobalVariables.add("slave_rows_search_algorithms");
        mysqlGlobalVariables.add("slave_skip_errors");
        mysqlGlobalVariables.add("slave_sql_verify_checksum");
        mysqlGlobalVariables.add("slave_transaction_retries");
        mysqlGlobalVariables.add("slave_type_conversions");
        mysqlGlobalVariables.add("slow_launch_time");
        mysqlGlobalVariables.add("slow_query_log");
        mysqlGlobalVariables.add("slow_query_log_file");
        mysqlGlobalVariables.add("socket");
        mysqlGlobalVariables.add("source_verify_checksum");
        mysqlGlobalVariables.add("sql_replica_skip_counter");
        mysqlGlobalVariables.add("sql_slave_skip_counter");
        mysqlGlobalVariables.add("ssl_ca");
        mysqlGlobalVariables.add("ssl_capath");
        mysqlGlobalVariables.add("ssl_cert");
        mysqlGlobalVariables.add("ssl_cipher");
        mysqlGlobalVariables.add("ssl_crl");
        mysqlGlobalVariables.add("ssl_crlpath");
        mysqlGlobalVariables.add("ssl_fips_mode");
        mysqlGlobalVariables.add("ssl_key");
        mysqlGlobalVariables.add("stored_program_cache");
        mysqlGlobalVariables.add("stored_program_definition_cache");
        mysqlGlobalVariables.add("super_read_only");
        mysqlGlobalVariables.add("sync_binlog");
        mysqlGlobalVariables.add("sync_master_info");
        mysqlGlobalVariables.add("sync_relay_log");
        mysqlGlobalVariables.add("sync_relay_log_info");
        mysqlGlobalVariables.add("sync_source_info");
        mysqlGlobalVariables.add("syseventlog.facility");
        mysqlGlobalVariables.add("syseventlog.include_pid");
        mysqlGlobalVariables.add("syseventlog.tag");
        mysqlGlobalVariables.add("system_time_zone");
        mysqlGlobalVariables.add("table_definition_cache");
        mysqlGlobalVariables.add("table_encryption_privilege_check");
        mysqlGlobalVariables.add("table_open_cache");
        mysqlGlobalVariables.add("table_open_cache_instances");
        mysqlGlobalVariables.add("tablespace_definition_cache");
        mysqlGlobalVariables.add("temptable_max_mmap");
        mysqlGlobalVariables.add("temptable_max_ram");
        mysqlGlobalVariables.add("temptable_use_mmap");
        mysqlGlobalVariables.add("thread_cache_size");
        mysqlGlobalVariables.add("thread_handling");
        mysqlGlobalVariables.add("thread_pool_algorithm");
        mysqlGlobalVariables.add("thread_pool_max_active_query_threads");
        mysqlGlobalVariables.add("thread_pool_max_unused_threads");
        mysqlGlobalVariables.add("thread_pool_prio_kickup_timer");
        mysqlGlobalVariables.add("thread_pool_size");
        mysqlGlobalVariables.add("thread_pool_stall_limit");
        mysqlGlobalVariables.add("thread_stack");
        mysqlGlobalVariables.add("tls_ciphersuites");
        mysqlGlobalVariables.add("tls_version");
        mysqlGlobalVariables.add("tmpdir");
        mysqlGlobalVariables.add("validate_password_check_user_name");
        mysqlGlobalVariables.add("validate_password_dictionary_file");
        mysqlGlobalVariables.add("validate_password_length");
        mysqlGlobalVariables.add("validate_password_mixed_case_count");
        mysqlGlobalVariables.add("validate_password_number_count");
        mysqlGlobalVariables.add("validate_password_policy");
        mysqlGlobalVariables.add("validate_password_special_char_count");
        mysqlGlobalVariables.add("validate_password.check_user_name");
        mysqlGlobalVariables.add("validate_password.dictionary_file");
        mysqlGlobalVariables.add("validate_password.length");
        mysqlGlobalVariables.add("validate_password.mixed_case_count");
        mysqlGlobalVariables.add("validate_password.number_count");
        mysqlGlobalVariables.add("validate_password.policy");
        mysqlGlobalVariables.add("validate_password.special_char_count");
        mysqlGlobalVariables.add("version");
        mysqlGlobalVariables.add("version_comment");
        mysqlGlobalVariables.add("version_compile_machine");
        mysqlGlobalVariables.add("version_compile_os");
        mysqlGlobalVariables.add("version_compile_zlib");
        mysqlSessionVariables.add("debug_sync");
        mysqlSessionVariables.add("error_count");
        mysqlSessionVariables.add("external_user");
        mysqlSessionVariables.add("gtid_next");
        mysqlSessionVariables.add("identity");
        mysqlSessionVariables.add("immediate_server_version");
        mysqlSessionVariables.add("innodb_parallel_read_threads");
        mysqlSessionVariables.add("insert_id");
        mysqlSessionVariables.add("last_insert_id");
        mysqlSessionVariables.add("mysqlx_read_timeout");
        mysqlSessionVariables.add("mysqlx_wait_timeout");
        mysqlSessionVariables.add("mysqlx_write_timeout");
        mysqlSessionVariables.add("ndb_table_no_logging");
        mysqlSessionVariables.add("ndb_table_temporary");
        mysqlSessionVariables.add("original_commit_timestamp");
        mysqlSessionVariables.add("original_server_version");
        mysqlSessionVariables.add("proxy_user");
        mysqlSessionVariables.add("pseudo_replica_mode");
        mysqlSessionVariables.add("pseudo_slave_mode");
        mysqlSessionVariables.add("pseudo_thread_id");
        mysqlSessionVariables.add("rand_seed1");
        mysqlSessionVariables.add("rand_seed2");
        mysqlSessionVariables.add("rbr_exec_mode");
        mysqlSessionVariables.add("require_row_format");
        mysqlSessionVariables.add("resultset_metadata");
        mysqlSessionVariables.add("secondary_engine_cost_threshold");
        mysqlSessionVariables.add("show_create_table_skip_secondary_engine");
        mysqlSessionVariables.add("sql_log_bin");
        mysqlSessionVariables.add("timestamp");
        mysqlSessionVariables.add("transaction_allow_batching");
        mysqlSessionVariables.add("use_secondary_engine");
        mysqlSessionVariables.add("warning_count");
        mysqlDynamicVariables.add("activate_all_roles_on_login");
        mysqlDynamicVariables.add("innodb_purge_history");
        mysqlDynamicVariables.add("replica_read_timeout");
        mysqlDynamicVariables.add("loose_innodb_prepare_wait_timeout");
        mysqlDynamicVariables.add("innodb_prepare_wait_timeout");
        mysqlDynamicVariables.add("admin_ssl_ca");
        mysqlDynamicVariables.add("admin_ssl_capath");
        mysqlDynamicVariables.add("admin_ssl_cert");
        mysqlDynamicVariables.add("admin_ssl_cipher");
        mysqlDynamicVariables.add("admin_ssl_crl");
        mysqlDynamicVariables.add("admin_ssl_crlpath");
        mysqlDynamicVariables.add("admin_ssl_key");
        mysqlDynamicVariables.add("admin_tls_ciphersuites");
        mysqlDynamicVariables.add("admin_tls_version");
        mysqlDynamicVariables.add("audit_log_connection_policy");
        mysqlDynamicVariables.add("audit_log_exclude_accounts");
        mysqlDynamicVariables.add("audit_log_flush");
        mysqlDynamicVariables.add("audit_log_format_unix_timestamp");
        mysqlDynamicVariables.add("audit_log_include_accounts");
        mysqlDynamicVariables.add("audit_log_max_size");
        mysqlDynamicVariables.add("audit_log_password_history_keep_days");
        mysqlDynamicVariables.add("audit_log_prune_seconds");
        mysqlDynamicVariables.add("audit_log_rotate_on_size");
        mysqlDynamicVariables.add("audit_log_statement_policy");
        mysqlDynamicVariables.add("authentication_kerberos_service_principal");
        mysqlDynamicVariables.add("authentication_ldap_sasl_auth_method_name");
        mysqlDynamicVariables.add("authentication_ldap_sasl_bind_base_dn");
        mysqlDynamicVariables.add("authentication_ldap_sasl_bind_root_dn");
        mysqlDynamicVariables.add("authentication_ldap_sasl_bind_root_pwd");
        mysqlDynamicVariables.add("authentication_ldap_sasl_ca_path");
        mysqlDynamicVariables.add("authentication_ldap_sasl_group_search_attr");
        mysqlDynamicVariables.add("authentication_ldap_sasl_group_search_filter");
        mysqlDynamicVariables.add("authentication_ldap_sasl_init_pool_size");
        mysqlDynamicVariables.add("authentication_ldap_sasl_log_status");
        mysqlDynamicVariables.add("authentication_ldap_sasl_max_pool_size");
        mysqlDynamicVariables.add("authentication_ldap_sasl_referral");
        mysqlDynamicVariables.add("authentication_ldap_sasl_server_host");
        mysqlDynamicVariables.add("authentication_ldap_sasl_server_port");
        mysqlDynamicVariables.add("authentication_ldap_sasl_tls");
        mysqlDynamicVariables.add("authentication_ldap_sasl_user_search_attr");
        mysqlDynamicVariables.add("authentication_ldap_simple_auth_method_name");
        mysqlDynamicVariables.add("authentication_ldap_simple_bind_base_dn");
        mysqlDynamicVariables.add("authentication_ldap_simple_bind_root_dn");
        mysqlDynamicVariables.add("authentication_ldap_simple_bind_root_pwd");
        mysqlDynamicVariables.add("authentication_ldap_simple_ca_path");
        mysqlDynamicVariables.add("authentication_ldap_simple_group_search_attr");
        mysqlDynamicVariables.add("authentication_ldap_simple_group_search_filter");
        mysqlDynamicVariables.add("authentication_ldap_simple_init_pool_size");
        mysqlDynamicVariables.add("authentication_ldap_simple_log_status");
        mysqlDynamicVariables.add("authentication_ldap_simple_max_pool_size");
        mysqlDynamicVariables.add("authentication_ldap_simple_referral");
        mysqlDynamicVariables.add("authentication_ldap_simple_server_host");
        mysqlDynamicVariables.add("authentication_ldap_simple_server_port");
        mysqlDynamicVariables.add("authentication_ldap_simple_tls");
        mysqlDynamicVariables.add("authentication_ldap_simple_user_search_attr");
        mysqlDynamicVariables.add("auto_increment_increment");
        mysqlDynamicVariables.add("auto_increment_offset");
        mysqlDynamicVariables.add("autocommit");
        mysqlDynamicVariables.add("automatic_sp_privileges");
        mysqlDynamicVariables.add("avoid_temporal_upgrade");
        mysqlDynamicVariables.add("big_tables");
        mysqlDynamicVariables.add("binlog_cache_size");
        mysqlDynamicVariables.add("binlog_checksum");
        mysqlDynamicVariables.add("binlog_direct_non_transactional_updates");
        mysqlDynamicVariables.add("binlog_encryption");
        mysqlDynamicVariables.add("binlog_error_action");
        mysqlDynamicVariables.add("binlog_expire_logs_seconds");
        mysqlDynamicVariables.add("binlog_format");
        mysqlDynamicVariables.add("binlog_group_commit_sync_delay");
        mysqlDynamicVariables.add("binlog_group_commit_sync_no_delay_count");
        mysqlDynamicVariables.add("binlog_max_flush_queue_time");
        mysqlDynamicVariables.add("binlog_order_commits");
        mysqlDynamicVariables.add("binlog_row_image");
        mysqlDynamicVariables.add("binlog_row_metadata");
        mysqlDynamicVariables.add("binlog_row_value_options");
        mysqlDynamicVariables.add("binlog_rows_query_log_events");
        mysqlDynamicVariables.add("binlog_stmt_cache_size");
        mysqlDynamicVariables.add("binlog_transaction_compression");
        mysqlDynamicVariables.add("binlog_transaction_compression_level_zstd");
        mysqlDynamicVariables.add("binlog_transaction_dependency_history_size");
        mysqlDynamicVariables.add("binlog_transaction_dependency_tracking");
        mysqlDynamicVariables.add("block_encryption_mode");
        mysqlDynamicVariables.add("bulk_insert_buffer_size");
        mysqlDynamicVariables.add("character_set_client");
        mysqlDynamicVariables.add("character_set_connection");
        mysqlDynamicVariables.add("character_set_database(note 1)");
        mysqlDynamicVariables.add("character_set_filesystem");
        mysqlDynamicVariables.add("character_set_results");
        mysqlDynamicVariables.add("character_set_server");
        mysqlDynamicVariables.add("check_proxy_users");
        mysqlDynamicVariables.add("clone_autotune_concurrency");
        mysqlDynamicVariables.add("clone_buffer_size");
        mysqlDynamicVariables.add("clone_ddl_timeout");
        mysqlDynamicVariables.add("clone_donor_timeout_after_network_failure");
        mysqlDynamicVariables.add("clone_enable_compression");
        mysqlDynamicVariables.add("clone_max_concurrency");
        mysqlDynamicVariables.add("clone_max_data_bandwidth");
        mysqlDynamicVariables.add("clone_max_network_bandwidth");
        mysqlDynamicVariables.add("clone_ssl_ca");
        mysqlDynamicVariables.add("clone_ssl_cert");
        mysqlDynamicVariables.add("clone_ssl_key");
        mysqlDynamicVariables.add("clone_valid_donor_list");
        mysqlDynamicVariables.add("collation_connection");
        mysqlDynamicVariables.add("collation_database (note 1)");
        mysqlDynamicVariables.add("collation_server");
        mysqlDynamicVariables.add("completion_type");
        mysqlDynamicVariables.add("concurrent_insert");
        mysqlDynamicVariables.add("connect_timeout");
        mysqlDynamicVariables.add("connection_control_failed_connections_threshold");
        mysqlDynamicVariables.add("connection_control_max_connection_delay");
        mysqlDynamicVariables.add("connection_control_min_connection_delay");
        mysqlDynamicVariables.add("cte_max_recursion_depth");
        mysqlDynamicVariables.add("debug");
        mysqlDynamicVariables.add("debug_sync");
        mysqlDynamicVariables.add("default_collation_for_utf8mb4");
        mysqlDynamicVariables.add("default_password_lifetime");
        mysqlDynamicVariables.add("default_storage_engine");
        mysqlDynamicVariables.add("default_table_encryption");
        mysqlDynamicVariables.add("default_tmp_storage_engine");
        mysqlDynamicVariables.add("default_week_format");
        mysqlDynamicVariables.add("delay_key_write");
        mysqlDynamicVariables.add("delayed_insert_limit");
        mysqlDynamicVariables.add("delayed_insert_timeout");
        mysqlDynamicVariables.add("delayed_queue_size");
        mysqlDynamicVariables.add("div_precision_increment");
        mysqlDynamicVariables.add("dragnet.log_error_filter_rules");
        mysqlDynamicVariables.add("end_markers_in_json");
        mysqlDynamicVariables.add("enforce_gtid_consistency");
        mysqlDynamicVariables.add("eq_range_index_dive_limit");
        mysqlDynamicVariables.add("event_scheduler");
        mysqlDynamicVariables.add("expire_logs_days");
        mysqlDynamicVariables.add("explicit_defaults_for_timestamp");
        mysqlDynamicVariables.add("flush");
        mysqlDynamicVariables.add("flush_time");
        mysqlDynamicVariables.add("foreign_key_checks");
        mysqlDynamicVariables.add("ft_boolean_syntax");
        mysqlDynamicVariables.add("general_log");
        mysqlDynamicVariables.add("general_log_file");
        mysqlDynamicVariables.add("generated_random_password_length");
        mysqlDynamicVariables.add("group_concat_max_len");
        mysqlDynamicVariables.add("group_replication_advertise_recovery_endpoints");
        mysqlDynamicVariables.add("group_replication_allow_local_lower_version_join");
        mysqlDynamicVariables.add("group_replication_auto_increment_increment");
        mysqlDynamicVariables.add("group_replication_autorejoin_tries");
        mysqlDynamicVariables.add("group_replication_bootstrap_group");
        mysqlDynamicVariables.add("group_replication_clone_threshold");
        mysqlDynamicVariables.add("group_replication_communication_debug_options");
        mysqlDynamicVariables.add("group_replication_communication_max_message_size");
        mysqlDynamicVariables.add("group_replication_components_stop_timeout");
        mysqlDynamicVariables.add("group_replication_compression_threshold");
        mysqlDynamicVariables.add("group_replication_consistency");
        mysqlDynamicVariables.add("group_replication_enforce_update_everywhere_checks");
        mysqlDynamicVariables.add("group_replication_exit_state_action");
        mysqlDynamicVariables.add("group_replication_flow_control_applier_threshold");
        mysqlDynamicVariables.add("group_replication_flow_control_certifier_threshold");
        mysqlDynamicVariables.add("group_replication_flow_control_hold_percent");
        mysqlDynamicVariables.add("group_replication_flow_control_max_commit_quota");
        mysqlDynamicVariables.add("group_replication_flow_control_member_quota_percent");
        mysqlDynamicVariables.add("group_replication_flow_control_min_quota");
        mysqlDynamicVariables.add("group_replication_flow_control_min_recovery_quota");
        mysqlDynamicVariables.add("group_replication_flow_control_mode");
        mysqlDynamicVariables.add("group_replication_flow_control_period");
        mysqlDynamicVariables.add("group_replication_flow_control_release_percent");
        mysqlDynamicVariables.add("group_replication_force_members");
        mysqlDynamicVariables.add("group_replication_group_name");
        mysqlDynamicVariables.add("group_replication_group_seeds");
        mysqlDynamicVariables.add("group_replication_gtid_assignment_block_size");
        mysqlDynamicVariables.add("group_replication_ip_allowlist");
        mysqlDynamicVariables.add("group_replication_ip_whitelist");
        mysqlDynamicVariables.add("group_replication_local_address");
        mysqlDynamicVariables.add("group_replication_member_expel_timeout");
        mysqlDynamicVariables.add("group_replication_member_weight");
        mysqlDynamicVariables.add("group_replication_message_cache_size");
        mysqlDynamicVariables.add("group_replication_poll_spin_loops");
        mysqlDynamicVariables.add("group_replication_recovery_complete_at");
        mysqlDynamicVariables.add("group_replication_recovery_compression_algorithms");
        mysqlDynamicVariables.add("group_replication_recovery_get_public_key");
        mysqlDynamicVariables.add("group_replication_recovery_public_key_path");
        mysqlDynamicVariables.add("group_replication_recovery_reconnect_interval");
        mysqlDynamicVariables.add("group_replication_recovery_retry_count");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_ca");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_capath");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_cert");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_cipher");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_crl");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_crlpath");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_key");
        mysqlDynamicVariables.add("group_replication_recovery_ssl_verify_server_cert");
        mysqlDynamicVariables.add("group_replication_recovery_tls_ciphersuites");
        mysqlDynamicVariables.add("group_replication_recovery_tls_version");
        mysqlDynamicVariables.add("group_replication_recovery_use_ssl");
        mysqlDynamicVariables.add("group_replication_recovery_zstd_compression_level");
        mysqlDynamicVariables.add("group_replication_single_primary_mode");
        mysqlDynamicVariables.add("group_replication_ssl_mode");
        mysqlDynamicVariables.add("group_replication_start_on_boot");
        mysqlDynamicVariables.add("group_replication_tls_source");
        mysqlDynamicVariables.add("group_replication_transaction_size_limit");
        mysqlDynamicVariables.add("group_replication_unreachable_majority_timeout");
        mysqlDynamicVariables.add("group_replication_view_change_uuid");
        mysqlDynamicVariables.add("gtid_executed_compression_period");
        mysqlDynamicVariables.add("gtid_mode");
        mysqlDynamicVariables.add("gtid_next");
        mysqlDynamicVariables.add("gtid_purged");
        mysqlDynamicVariables.add("histogram_generation_max_mem_size");
        mysqlDynamicVariables.add("host_cache_size");
        mysqlDynamicVariables.add("identity");
        mysqlDynamicVariables.add("immediate_server_version");
        mysqlDynamicVariables.add("information_schema_stats_expiry");
        mysqlDynamicVariables.add("init_connect");
        mysqlDynamicVariables.add("init_replica");
        mysqlDynamicVariables.add("init_slave");
        mysqlDynamicVariables.add("innodb_adaptive_flushing");
        mysqlDynamicVariables.add("innodb_adaptive_flushing_lwm");
        mysqlDynamicVariables.add("innodb_adaptive_hash_index");
        mysqlDynamicVariables.add("innodb_adaptive_max_sleep_delay");
        mysqlDynamicVariables.add("innodb_api_bk_commit_interval");
        mysqlDynamicVariables.add("innodb_api_trx_level");
        mysqlDynamicVariables.add("innodb_autoextend_increment");
        mysqlDynamicVariables.add("innodb_background_drop_list_empty");
        mysqlDynamicVariables.add("innodb_buffer_pool_dump_at_shutdown");
        mysqlDynamicVariables.add("innodb_buffer_pool_dump_now");
        mysqlDynamicVariables.add("innodb_buffer_pool_dump_pct");
        mysqlDynamicVariables.add("innodb_buffer_pool_filename");
        mysqlDynamicVariables.add("innodb_buffer_pool_in_core_file");
        mysqlDynamicVariables.add("innodb_buffer_pool_load_abort");
        mysqlDynamicVariables.add("innodb_buffer_pool_load_now");
        mysqlDynamicVariables.add("innodb_buffer_pool_size");
        mysqlDynamicVariables.add("innodb_change_buffer_max_size");
        mysqlDynamicVariables.add("innodb_change_buffering");
        mysqlDynamicVariables.add("innodb_change_buffering_debug");
        mysqlDynamicVariables.add("innodb_checkpoint_disabled");
        mysqlDynamicVariables.add("innodb_checksum_algorithm");
        mysqlDynamicVariables.add("innodb_cmp_per_index_enabled");
        mysqlDynamicVariables.add("innodb_commit_concurrency");
        mysqlDynamicVariables.add("innodb_compress_debug");
        mysqlDynamicVariables.add("innodb_compression_failure_threshold_pct");
        mysqlDynamicVariables.add("innodb_compression_level");
        mysqlDynamicVariables.add("innodb_compression_pad_pct_max");
        mysqlDynamicVariables.add("innodb_concurrency_tickets");
        mysqlDynamicVariables.add("innodb_data_file_purge");
        mysqlDynamicVariables.add("innodb_data_file_purge_interval");
        mysqlDynamicVariables.add("innodb_data_file_purge_max_size");
        mysqlDynamicVariables.add("innodb_ddl_log_crash_reset_debug");
        mysqlDynamicVariables.add("innodb_deadlock_detect");
        mysqlDynamicVariables.add("innodb_default_row_format");
        mysqlDynamicVariables.add("innodb_disable_sort_file_cache");
        mysqlDynamicVariables.add("innodb_extend_and_initialize");
        mysqlDynamicVariables.add("innodb_fast_shutdown");
        mysqlDynamicVariables.add("innodb_fil_make_page_dirty_debug");
        mysqlDynamicVariables.add("innodb_file_per_table");
        mysqlDynamicVariables.add("innodb_fill_factor");
        mysqlDynamicVariables.add("innodb_flush_log_at_timeout");
        mysqlDynamicVariables.add("innodb_flush_log_at_trx_commit");
        mysqlDynamicVariables.add("innodb_flush_neighbors");
        mysqlDynamicVariables.add("innodb_flush_sync");
        mysqlDynamicVariables.add("innodb_flushing_avg_loops");
        mysqlDynamicVariables.add("innodb_fsync_threshold");
        mysqlDynamicVariables.add("innodb_ft_aux_table");
        mysqlDynamicVariables.add("innodb_ft_enable_diag_print");
        mysqlDynamicVariables.add("innodb_ft_enable_stopword");
        mysqlDynamicVariables.add("innodb_ft_num_word_optimize");
        mysqlDynamicVariables.add("innodb_ft_result_cache_limit");
        mysqlDynamicVariables.add("innodb_ft_server_stopword_table");
        mysqlDynamicVariables.add("innodb_ft_user_stopword_table");
        mysqlDynamicVariables.add("innodb_idle_flush_pct");
        mysqlDynamicVariables.add("innodb_io_capacity");
        mysqlDynamicVariables.add("innodb_io_capacity_max");
        mysqlDynamicVariables.add("innodb_limit_optimistic_insert_debug");
        mysqlDynamicVariables.add("innodb_lock_wait_timeout");
        mysqlDynamicVariables.add("innodb_log_checkpoint_fuzzy_now");
        mysqlDynamicVariables.add("innodb_log_checkpoint_now");
        mysqlDynamicVariables.add("innodb_log_checksums");
        mysqlDynamicVariables.add("innodb_log_compressed_pages");
        mysqlDynamicVariables.add("innodb_log_spin_cpu_abs_lwm");
        mysqlDynamicVariables.add("innodb_log_spin_cpu_pct_hwm");
        mysqlDynamicVariables.add("innodb_log_wait_for_flush_spin_hwm");
        mysqlDynamicVariables.add("innodb_log_write_ahead_size");
        mysqlDynamicVariables.add("innodb_log_writer_threads");
        mysqlDynamicVariables.add("innodb_lru_scan_depth");
        mysqlDynamicVariables.add("innodb_max_dirty_pages_pct");
        mysqlDynamicVariables.add("innodb_max_dirty_pages_pct_lwm");
        mysqlDynamicVariables.add("innodb_max_purge_lag");
        mysqlDynamicVariables.add("innodb_max_purge_lag_delay");
        mysqlDynamicVariables.add("innodb_max_undo_log_size");
        mysqlDynamicVariables.add("innodb_merge_threshold_set_all_debug");
        mysqlDynamicVariables.add("innodb_monitor_disable");
        mysqlDynamicVariables.add("innodb_monitor_enable");
        mysqlDynamicVariables.add("innodb_monitor_reset");
        mysqlDynamicVariables.add("innodb_monitor_reset_all");
        mysqlDynamicVariables.add("innodb_old_blocks_pct");
        mysqlDynamicVariables.add("innodb_old_blocks_time");
        mysqlDynamicVariables.add("innodb_online_alter_log_max_size");
        mysqlDynamicVariables.add("innodb_optimize_fulltext_only");
        mysqlDynamicVariables.add("innodb_parallel_read_threads");
        mysqlDynamicVariables.add("innodb_print_all_deadlocks");
        mysqlDynamicVariables.add("innodb_print_ddl_logs");
        mysqlDynamicVariables.add("innodb_purge_batch_size");
        mysqlDynamicVariables.add("innodb_purge_rseg_truncate_frequency");
        mysqlDynamicVariables.add("innodb_random_read_ahead");
        mysqlDynamicVariables.add("innodb_read_ahead_threshold");
        mysqlDynamicVariables.add("innodb_redo_log_archive_dirs");
        mysqlDynamicVariables.add("innodb_redo_log_encrypt");
        mysqlDynamicVariables.add("innodb_replication_delay");
        mysqlDynamicVariables.add("innodb_rollback_segments");
        mysqlDynamicVariables.add("innodb_saved_page_number_debug");
        mysqlDynamicVariables.add("innodb_segment_reserve_factor");
        mysqlDynamicVariables.add("innodb_spin_wait_delay");
        mysqlDynamicVariables.add("innodb_spin_wait_pause_multiplier");
        mysqlDynamicVariables.add("innodb_stats_auto_recalc");
        mysqlDynamicVariables.add("innodb_stats_include_delete_marked");
        mysqlDynamicVariables.add("innodb_stats_method");
        mysqlDynamicVariables.add("innodb_stats_on_metadata");
        mysqlDynamicVariables.add("innodb_stats_persistent");
        mysqlDynamicVariables.add("innodb_stats_persistent_sample_pages");
        mysqlDynamicVariables.add("innodb_stats_transient_sample_pages");
        mysqlDynamicVariables.add("innodb_status_output");
        mysqlDynamicVariables.add("innodb_status_output_locks");
        mysqlDynamicVariables.add("innodb_strict_mode");
        mysqlDynamicVariables.add("innodb_support_instant_add_column");
        mysqlDynamicVariables.add("innodb_sync_spin_loops");
        mysqlDynamicVariables.add("innodb_table_locks");
        mysqlDynamicVariables.add("innodb_thread_concurrency");
        mysqlDynamicVariables.add("innodb_thread_sleep_delay");
        mysqlDynamicVariables.add("innodb_tmpdir");
        mysqlDynamicVariables.add("innodb_trx_purge_view_update_only_debug");
        mysqlDynamicVariables.add("innodb_trx_rseg_n_slots_debug");
        mysqlDynamicVariables.add("innodb_undo_log_encrypt");
        mysqlDynamicVariables.add("innodb_undo_log_truncate");
        mysqlDynamicVariables.add("innodb_use_fdatasync");
        mysqlDynamicVariables.add("insert_id");
        mysqlDynamicVariables.add("interactive_timeout");
        mysqlDynamicVariables.add("internal_tmp_disk_storage_engine");
        mysqlDynamicVariables.add("internal_tmp_mem_storage_engine");
        mysqlDynamicVariables.add("join_buffer_size");
        mysqlDynamicVariables.add("keep_files_on_create");
        mysqlDynamicVariables.add("key_buffer_size");
        mysqlDynamicVariables.add("key_cache_age_threshold");
        mysqlDynamicVariables.add("key_cache_block_size");
        mysqlDynamicVariables.add("key_cache_division_limit");
        mysqlDynamicVariables.add("keyring_aws_cmk_id");
        mysqlDynamicVariables.add("keyring_aws_region");
        mysqlDynamicVariables.add("keyring_encrypted_file_data");
        mysqlDynamicVariables.add("keyring_encrypted_file_password");
        mysqlDynamicVariables.add("keyring_file_data");
        mysqlDynamicVariables.add("keyring_hashicorp_auth_path");
        mysqlDynamicVariables.add("keyring_hashicorp_ca_path");
        mysqlDynamicVariables.add("keyring_hashicorp_caching");
        mysqlDynamicVariables.add("keyring_hashicorp_role_id");
        mysqlDynamicVariables.add("keyring_hashicorp_secret_id");
        mysqlDynamicVariables.add("keyring_hashicorp_server_url");
        mysqlDynamicVariables.add("keyring_hashicorp_store_path");
        mysqlDynamicVariables.add("keyring_okv_conf_dir");
        mysqlDynamicVariables.add("keyring_operations");
        mysqlDynamicVariables.add("last_insert_id");
        mysqlDynamicVariables.add("lc_messages");
        mysqlDynamicVariables.add("lc_time_names");
        mysqlDynamicVariables.add("local_infile");
        mysqlDynamicVariables.add("lock_wait_timeout");
        mysqlDynamicVariables.add("log_bin_trust_function_creators");
        mysqlDynamicVariables.add("log_bin_use_v1_row_events");
        mysqlDynamicVariables.add("log_error_services");
        mysqlDynamicVariables.add("log_error_suppression_list");
        mysqlDynamicVariables.add("log_error_verbosity");
        mysqlDynamicVariables.add("log_output");
        mysqlDynamicVariables.add("log_queries_not_using_indexes");
        mysqlDynamicVariables.add("log_raw");
        mysqlDynamicVariables.add("log_slow_admin_statements");
        mysqlDynamicVariables.add("log_slow_extra");
        mysqlDynamicVariables.add("log_slow_replica_statements");
        mysqlDynamicVariables.add("log_slow_slave_statements");
        mysqlDynamicVariables.add("log_statements_unsafe_for_binlog");
        mysqlDynamicVariables.add("log_syslog");
        mysqlDynamicVariables.add("log_syslog_facility");
        mysqlDynamicVariables.add("log_syslog_include_pid");
        mysqlDynamicVariables.add("log_syslog_tag");
        mysqlDynamicVariables.add("log_throttle_queries_not_using_indexes");
        mysqlDynamicVariables.add("log_timestamps");
        mysqlDynamicVariables.add("long_query_time");
        mysqlDynamicVariables.add("low_priority_updates");
        mysqlDynamicVariables.add("mandatory_roles");
        mysqlDynamicVariables.add("master_info_repository");
        mysqlDynamicVariables.add("master_verify_checksum");
        mysqlDynamicVariables.add("max_allowed_packet");
        mysqlDynamicVariables.add("max_binlog_cache_size");
        mysqlDynamicVariables.add("max_binlog_size");
        mysqlDynamicVariables.add("max_binlog_stmt_cache_size");
        mysqlDynamicVariables.add("max_connect_errors");
        mysqlDynamicVariables.add("max_connections");
        mysqlDynamicVariables.add("max_delayed_threads");
        mysqlDynamicVariables.add("max_error_count");
        mysqlDynamicVariables.add("max_execution_time");
        mysqlDynamicVariables.add("max_heap_table_size");
        mysqlDynamicVariables.add("max_insert_delayed_threads");
        mysqlDynamicVariables.add("max_join_size");
        mysqlDynamicVariables.add("max_length_for_sort_data");
        mysqlDynamicVariables.add("max_points_in_geometry");
        mysqlDynamicVariables.add("max_prepared_stmt_count");
        mysqlDynamicVariables.add("max_relay_log_size");
        mysqlDynamicVariables.add("max_seeks_for_key");
        mysqlDynamicVariables.add("max_sort_length");
        mysqlDynamicVariables.add("max_sp_recursion_depth");
        mysqlDynamicVariables.add("max_user_connections");
        mysqlDynamicVariables.add("max_write_lock_count");
        mysqlDynamicVariables.add("min_examined_row_limit");
        mysqlDynamicVariables.add("myisam_data_pointer_size");
        mysqlDynamicVariables.add("myisam_max_sort_file_size");
        mysqlDynamicVariables.add("myisam_repair_threads");
        mysqlDynamicVariables.add("myisam_sort_buffer_size");
        mysqlDynamicVariables.add("myisam_stats_method");
        mysqlDynamicVariables.add("myisam_use_mmap");
        mysqlDynamicVariables.add("mysql_firewall_mode");
        mysqlDynamicVariables.add("mysql_firewall_trace");
        mysqlDynamicVariables.add("mysql_native_password_proxy_users");
        mysqlDynamicVariables.add("mysqlx_compression_algorithms");
        mysqlDynamicVariables.add("mysqlx_connect_timeout");
        mysqlDynamicVariables.add("mysqlx_deflate_default_compression_level");
        mysqlDynamicVariables.add("mysqlx_deflate_max_client_compression_level");
        mysqlDynamicVariables.add("mysqlx_document_id_unique_prefix");
        mysqlDynamicVariables.add("mysqlx_enable_hello_notice");
        mysqlDynamicVariables.add("mysqlx_idle_worker_thread_timeout");
        mysqlDynamicVariables.add("mysqlx_interactive_timeout");
        mysqlDynamicVariables.add("mysqlx_lz4_default_compression_level");
        mysqlDynamicVariables.add("mysqlx_lz4_max_client_compression_level");
        mysqlDynamicVariables.add("mysqlx_max_allowed_packet");
        mysqlDynamicVariables.add("mysqlx_max_connections");
        mysqlDynamicVariables.add("mysqlx_min_worker_threads");
        mysqlDynamicVariables.add("mysqlx_read_timeout");
        mysqlDynamicVariables.add("mysqlx_wait_timeout");
        mysqlDynamicVariables.add("mysqlx_write_timeout");
        mysqlDynamicVariables.add("mysqlx_zstd_default_compression_level");
        mysqlDynamicVariables.add("mysqlx_zstd_max_client_compression_level");
        mysqlDynamicVariables.add("ndb_allow_copying_alter_table");
        mysqlDynamicVariables.add("ndb_autoincrement_prefetch_sz");
        mysqlDynamicVariables.add("ndb_blob_read_batch_bytes");
        mysqlDynamicVariables.add("ndb_blob_write_batch_bytes");
        mysqlDynamicVariables.add("ndb_cache_check_time");
        mysqlDynamicVariables.add("ndb_clear_apply_status");
        mysqlDynamicVariables.add("ndb_conflict_role");
        mysqlDynamicVariables.add("ndb_data_node_neighbour");
        mysqlDynamicVariables.add("ndb_dbg_check_shares");
        mysqlDynamicVariables.add("ndb_default_column_format");
        mysqlDynamicVariables.add("ndb_default_column_format");
        mysqlDynamicVariables.add("ndb_deferred_constraints");
        mysqlDynamicVariables.add("ndb_deferred_constraints");
        mysqlDynamicVariables.add("ndb_distribution");
        mysqlDynamicVariables.add("ndb_distribution");
        mysqlDynamicVariables.add("ndb_eventbuffer_free_percent");
        mysqlDynamicVariables.add("ndb_eventbuffer_max_alloc");
        mysqlDynamicVariables.add("ndb_extra_logging");
        mysqlDynamicVariables.add("ndb_force_send");
        mysqlDynamicVariables.add("ndb_fully_replicated");
        mysqlDynamicVariables.add("ndb_index_stat_enable");
        mysqlDynamicVariables.add("ndb_index_stat_option");
        mysqlDynamicVariables.add("ndb_join_pushdown");
        mysqlDynamicVariables.add("ndb_log_binlog_index");
        mysqlDynamicVariables.add("ndb_log_empty_epochs");
        mysqlDynamicVariables.add("ndb_log_empty_epochs");
        mysqlDynamicVariables.add("ndb_log_empty_update");
        mysqlDynamicVariables.add("ndb_log_empty_update");
        mysqlDynamicVariables.add("ndb_log_exclusive_reads");
        mysqlDynamicVariables.add("ndb_log_exclusive_reads");
        mysqlDynamicVariables.add("ndb_log_update_as_write");
        mysqlDynamicVariables.add("ndb_log_update_minimal");
        mysqlDynamicVariables.add("ndb_log_updated_only");
        mysqlDynamicVariables.add("ndb_metadata_check");
        mysqlDynamicVariables.add("ndb_metadata_check_interval");
        mysqlDynamicVariables.add("ndb_metadata_sync");
        mysqlDynamicVariables.add("ndb_optimization_delay");
        mysqlDynamicVariables.add("ndb_read_backup");
        mysqlDynamicVariables.add("ndb_recv_thread_activation_threshold");
        mysqlDynamicVariables.add("ndb_recv_thread_cpu_mask");
        mysqlDynamicVariables.add("ndb_report_thresh_binlog_epoch_slip");
        mysqlDynamicVariables.add("ndb_report_thresh_binlog_mem_usage");
        mysqlDynamicVariables.add("ndb_row_checksum");
        mysqlDynamicVariables.add("ndb_schema_dist_lock_wait_timeout");
        mysqlDynamicVariables.add("ndb_show_foreign_key_mock_tables");
        mysqlDynamicVariables.add("ndb_slave_conflict_role");
        mysqlDynamicVariables.add("ndb_table_no_logging");
        mysqlDynamicVariables.add("ndb_table_temporary");
        mysqlDynamicVariables.add("ndb_use_exact_count");
        mysqlDynamicVariables.add("ndb_use_transactions");
        mysqlDynamicVariables.add("ndbinfo_max_bytes");
        mysqlDynamicVariables.add("ndbinfo_max_rows");
        mysqlDynamicVariables.add("ndbinfo_offline");
        mysqlDynamicVariables.add("ndbinfo_show_hidden");
        mysqlDynamicVariables.add("net_buffer_length");
        mysqlDynamicVariables.add("net_read_timeout");
        mysqlDynamicVariables.add("net_retry_count");
        mysqlDynamicVariables.add("net_write_timeout");
        mysqlDynamicVariables.add("new");
        mysqlDynamicVariables.add("offline_mode");
        mysqlDynamicVariables.add("old_alter_table");
        mysqlDynamicVariables.add("optimizer_prune_level");
        mysqlDynamicVariables.add("optimizer_search_depth");
        mysqlDynamicVariables.add("optimizer_switch");
        mysqlDynamicVariables.add("optimizer_trace");
        mysqlDynamicVariables.add("optimizer_trace_features");
        mysqlDynamicVariables.add("optimizer_trace_limit");
        mysqlDynamicVariables.add("optimizer_trace_max_mem_size");
        mysqlDynamicVariables.add("optimizer_trace_offset");
        mysqlDynamicVariables.add("original_commit_timestamp");
        mysqlDynamicVariables.add("original_server_version");
        mysqlDynamicVariables.add("parser_max_mem_size");
        mysqlDynamicVariables.add("partial_revokes");
        mysqlDynamicVariables.add("password_history");
        mysqlDynamicVariables.add("password_require_current");
        mysqlDynamicVariables.add("password_reuse_interval");
        mysqlDynamicVariables.add("performance_schema_max_digest_sample_age");
        mysqlDynamicVariables.add("performance_schema_show_processlist");
        mysqlDynamicVariables.add("preload_buffer_size");
        mysqlDynamicVariables.add("print_identified_with_as_hex");
        mysqlDynamicVariables.add("profiling");
        mysqlDynamicVariables.add("profiling_history_size");
        mysqlDynamicVariables.add("protocol_compression_algorithms");
        mysqlDynamicVariables.add("pseudo_replica_mode");
        mysqlDynamicVariables.add("pseudo_slave_mode");
        mysqlDynamicVariables.add("pseudo_thread_id");
        mysqlDynamicVariables.add("query_alloc_block_size");
        mysqlDynamicVariables.add("query_prealloc_size");
        mysqlDynamicVariables.add("rand_seed1");
        mysqlDynamicVariables.add("rand_seed2");
        mysqlDynamicVariables.add("range_alloc_block_size");
        mysqlDynamicVariables.add("range_optimizer_max_mem_size");
        mysqlDynamicVariables.add("polarx_max_allowed_packet");
        mysqlDynamicVariables.add("polarx_max_connections");
        mysqlDynamicVariables.add("polarx_connect_timeout");
        mysqlDynamicVariables.add("rbr_exec_mode");
        mysqlDynamicVariables.add("read_buffer_size");
        mysqlDynamicVariables.add("read_only");
        mysqlDynamicVariables.add("read_rnd_buffer_size");
        mysqlDynamicVariables.add("regexp_stack_limit");
        mysqlDynamicVariables.add("regexp_time_limit");
        mysqlDynamicVariables.add("relay_log_info_repository");
        mysqlDynamicVariables.add("relay_log_purge");
        mysqlDynamicVariables.add("replica_allow_batching");
        mysqlDynamicVariables.add("replica_checkpoint_group");
        mysqlDynamicVariables.add("replica_checkpoint_period");
        mysqlDynamicVariables.add("replica_compressed_protocol");
        mysqlDynamicVariables.add("replica_exec_mode");
        mysqlDynamicVariables.add("replica_max_allowed_packet");
        mysqlDynamicVariables.add("replica_net_timeout");
        mysqlDynamicVariables.add("replica_parallel_type");
        mysqlDynamicVariables.add("replica_parallel_workers");
        mysqlDynamicVariables.add("replica_pending_jobs_size_max");
        mysqlDynamicVariables.add("replica_preserve_commit_order");
        mysqlDynamicVariables.add("replica_sql_verify_checksum");
        mysqlDynamicVariables.add("replica_transaction_retries");
        mysqlDynamicVariables.add("replica_type_conversions");
        mysqlDynamicVariables.add("replication_optimize_for_static_plugin_config");
        mysqlDynamicVariables.add("replication_sender_observe_commit_only");
        mysqlDynamicVariables.add("require_row_format");
        mysqlDynamicVariables.add("require_secure_transport");
        mysqlDynamicVariables.add("resultset_metadata");
        mysqlDynamicVariables.add("rewriter_enabled");
        mysqlDynamicVariables.add("rewriter_verbose");
        mysqlDynamicVariables.add("rpl_read_size");
        mysqlDynamicVariables.add("rpl_semi_sync_master_enabled");
        mysqlDynamicVariables.add("rpl_semi_sync_master_timeout");
        mysqlDynamicVariables.add("rpl_semi_sync_master_trace_level");
        mysqlDynamicVariables.add("rpl_semi_sync_master_wait_for_slave_count");
        mysqlDynamicVariables.add("rpl_semi_sync_master_wait_no_slave");
        mysqlDynamicVariables.add("rpl_semi_sync_master_wait_point");
        mysqlDynamicVariables.add("rpl_semi_sync_replica_enabled");
        mysqlDynamicVariables.add("rpl_semi_sync_replica_trace_level");
        mysqlDynamicVariables.add("rpl_semi_sync_slave_enabled");
        mysqlDynamicVariables.add("rpl_semi_sync_slave_trace_level");
        mysqlDynamicVariables.add("rpl_semi_sync_source_enabled");
        mysqlDynamicVariables.add("rpl_semi_sync_source_timeout");
        mysqlDynamicVariables.add("rpl_semi_sync_source_trace_level");
        mysqlDynamicVariables.add("rpl_semi_sync_source_wait_for_replica_count");
        mysqlDynamicVariables.add("rpl_semi_sync_source_wait_no_replica");
        mysqlDynamicVariables.add("rpl_semi_sync_source_wait_point");
        mysqlDynamicVariables.add("rpl_stop_replica_timeout");
        mysqlDynamicVariables.add("rpl_stop_slave_timeout");
        mysqlDynamicVariables.add("schema_definition_cache");
        mysqlDynamicVariables.add("secondary_engine_cost_threshold");
        mysqlDynamicVariables.add("select_into_buffer_size");
        mysqlDynamicVariables.add("select_into_disk_sync");
        mysqlDynamicVariables.add("select_into_disk_sync_delay");
        mysqlDynamicVariables.add("server_id");
        mysqlDynamicVariables.add("session_track_gtids");
        mysqlDynamicVariables.add("session_track_schema");
        mysqlDynamicVariables.add("session_track_state_change");
        mysqlDynamicVariables.add("session_track_system_variables");
        mysqlDynamicVariables.add("session_track_transaction_info");
        mysqlDynamicVariables.add("sha256_password_proxy_users");
        mysqlDynamicVariables.add("show_create_table_skip_secondary_engine");
        mysqlDynamicVariables.add("show_create_table_verbosity");
        mysqlDynamicVariables.add("show_old_temporals");
        mysqlDynamicVariables.add("slave_allow_batching");
        mysqlDynamicVariables.add("slave_checkpoint_group");
        mysqlDynamicVariables.add("slave_checkpoint_period");
        mysqlDynamicVariables.add("slave_compressed_protocol");
        mysqlDynamicVariables.add("slave_exec_mode");
        mysqlDynamicVariables.add("slave_max_allowed_packet");
        mysqlDynamicVariables.add("slave_net_timeout");
        mysqlDynamicVariables.add("slave_parallel_type");
        mysqlDynamicVariables.add("slave_parallel_workers");
        mysqlDynamicVariables.add("slave_pending_jobs_size_max");
        mysqlDynamicVariables.add("slave_preserve_commit_order");
        mysqlDynamicVariables.add("slave_rows_search_algorithms");
        mysqlDynamicVariables.add("slave_sql_verify_checksum");
        mysqlDynamicVariables.add("slave_transaction_retries");
        mysqlDynamicVariables.add("slave_type_conversions");
        mysqlDynamicVariables.add("slow_launch_time");
        mysqlDynamicVariables.add("slow_query_log");
        mysqlDynamicVariables.add("slow_query_log_file");
        mysqlDynamicVariables.add("sort_buffer_size");
        mysqlDynamicVariables.add("source_verify_checksum");
        mysqlDynamicVariables.add("sql_auto_is_null");
        mysqlDynamicVariables.add("sql_big_selects");
        mysqlDynamicVariables.add("sql_buffer_result");
        mysqlDynamicVariables.add("sql_log_bin");
        mysqlDynamicVariables.add("sql_log_off");
        mysqlDynamicVariables.add("sql_mode");
        mysqlDynamicVariables.add("sql_notes");
        mysqlDynamicVariables.add("sql_quote_show_create");
        mysqlDynamicVariables.add("sql_replica_skip_counter");
        mysqlDynamicVariables.add("sql_require_primary_key");
        mysqlDynamicVariables.add("sql_safe_updates");
        mysqlDynamicVariables.add("sql_select_limit");
        mysqlDynamicVariables.add("sql_slave_skip_counter");
        mysqlDynamicVariables.add("sql_warnings");
        mysqlDynamicVariables.add("ssl_fips_mode");
        mysqlDynamicVariables.add("stored_program_cache");
        mysqlDynamicVariables.add("stored_program_definition_cache");
        mysqlDynamicVariables.add("super_read_only");
        mysqlDynamicVariables.add("sync_binlog");
        mysqlDynamicVariables.add("sync_master_info");
        mysqlDynamicVariables.add("sync_relay_log");
        mysqlDynamicVariables.add("sync_relay_log_info");
        mysqlDynamicVariables.add("sync_source_info");
        mysqlDynamicVariables.add("syseventlog.facility");
        mysqlDynamicVariables.add("syseventlog.include_pid");
        mysqlDynamicVariables.add("syseventlog.tag");
        mysqlDynamicVariables.add("table_definition_cache");
        mysqlDynamicVariables.add("table_encryption_privilege_check");
        mysqlDynamicVariables.add("table_open_cache");
        mysqlDynamicVariables.add("tablespace_definition_cache");
        mysqlDynamicVariables.add("temptable_max_mmap");
        mysqlDynamicVariables.add("temptable_max_ram");
        mysqlDynamicVariables.add("temptable_use_mmap");
        mysqlDynamicVariables.add("terminology_use_previous");
        mysqlDynamicVariables.add("thread_cache_size");
        mysqlDynamicVariables.add("thread_pool_high_priority_connection");
        mysqlDynamicVariables.add("thread_pool_max_active_query_threads");
        mysqlDynamicVariables.add("thread_pool_max_unused_threads");
        mysqlDynamicVariables.add("thread_pool_prio_kickup_timer");
        mysqlDynamicVariables.add("thread_pool_stall_limit");
        mysqlDynamicVariables.add("time_zone");
        mysqlDynamicVariables.add("timestamp");
        mysqlDynamicVariables.add("tls_ciphersuites");
        mysqlDynamicVariables.add("tmp_table_size");
        mysqlDynamicVariables.add("transaction_alloc_block_size");
        mysqlDynamicVariables.add("transaction_allow_batching");
        mysqlDynamicVariables.add("transaction_isolation");
        mysqlDynamicVariables.add("transaction_prealloc_size");
        mysqlDynamicVariables.add("transaction_read_only");
        mysqlDynamicVariables.add("transaction_write_set_extraction");
        mysqlDynamicVariables.add("unique_checks");
        mysqlDynamicVariables.add("updatable_views_with_limit");
        mysqlDynamicVariables.add("use_secondary_engine");
        mysqlDynamicVariables.add("validate_password_check_user_name");
        mysqlDynamicVariables.add("validate_password_dictionary_file");
        mysqlDynamicVariables.add("validate_password_length");
        mysqlDynamicVariables.add("validate_password_mixed_case_count");
        mysqlDynamicVariables.add("validate_password_number_count");
        mysqlDynamicVariables.add("validate_password_policy");
        mysqlDynamicVariables.add("validate_password_special_char_count");
        mysqlDynamicVariables.add("validate_password.check_user_name");
        mysqlDynamicVariables.add("validate_password.dictionary_file");
        mysqlDynamicVariables.add("validate_password.length");
        mysqlDynamicVariables.add("validate_password.mixed_case_count");
        mysqlDynamicVariables.add("validate_password.number_count");
        mysqlDynamicVariables.add("validate_password.policy");
        mysqlDynamicVariables.add("validate_password.special_char_count");
        mysqlDynamicVariables.add("version_tokens_session");
        mysqlDynamicVariables.add("wait_timeout");
        mysqlDynamicVariables.add("windowing_use_high_precision");

        globalBannedVariables.add("gtid_mode");

        globalBannedVariables.add("max_allowed_packet");
        globalBannedVariables.add("max_user_connections");
        globalBannedVariables.add("max_connections");
        globalBannedVariables.add("autocommmit");
        globalBannedVariables.add("read_only");
        globalBannedVariables.add("auto_increment_increment");

        ImmutableSet.Builder<String> modifiableTimerTaskVarBuilder = new ImmutableSet.Builder<>();
        ImmutableSet.Builder<String> supportSetGlobalVarBuilder = new ImmutableSet.Builder<>();

        for (String modifiableTimerTaskParam : MODIFIABLE_PURGE_TRANS_PARAM) {
            modifiableTimerTaskVarBuilder.add(modifiableTimerTaskParam);
            supportSetGlobalVarBuilder.add(modifiableTimerTaskParam.toLowerCase());
        }

        for (String modifiableTimerTaskParam : MODIFIABLE_DEADLOCK_DETECTION_PARAM) {
            modifiableTimerTaskVarBuilder.add(modifiableTimerTaskParam);
            supportSetGlobalVarBuilder.add(modifiableTimerTaskParam.toLowerCase());
        }

        MODIFIABLE_TIMER_TASK_PARAM = modifiableTimerTaskVarBuilder.build();

        SUPPORT_SET_GLOBAL_VARIABLES = supportSetGlobalVarBuilder.build();
    }

    public static boolean contains(String variable) {
        return variables.contains(variable.toLowerCase());
    }

    public static boolean isWritable(String variable) {
        return writableVariables.contains(variable.toLowerCase());
    }

    public static boolean isBanned(String variable) {
        return bannedVariables.contains(variable.toLowerCase());
    }

    public static boolean isReadonly(String variable) {
        return readonlyVariables.contains(variable.toLowerCase());
    }

    public static boolean isExtra(String variable) {
        return extraVariables.contains(variable.toLowerCase());
    }

    public static boolean isXBanned(String variable) {
        return XbannedVariables.contains(variable.toLowerCase());
    }

    public static boolean isMysqlBoth(String variable) {
        return mysqlBothVariables.contains(variable.toLowerCase());
    }

    public static boolean isMysqlGlobal(String variable) {
        return mysqlGlobalVariables.contains(variable.toLowerCase());
    }

    public static boolean isMysqlSession(String variable) {
        return mysqlSessionVariables.contains(variable.toLowerCase());
    }

    public static boolean isMysqlDynamic(String variable) {
        return mysqlDynamicVariables.contains(variable.toLowerCase());
    }

    public static boolean isGlobalBanned(String variable) {
        return globalBannedVariables.contains(variable.toLowerCase());
    }

    public static boolean isSetGlobalSupported(String variable) {
        return SUPPORT_SET_GLOBAL_VARIABLES.contains(variable.toLowerCase());
    }
}
